//======================================================================
// File:		bittableitem.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <qdrawutil.h>
#include <qpainter.h>

#include <bittableitem.h>

BitTableItem::BitTableItem( QTable *table,  const QString & text ) :
  QTableItem( table, QTableItem::Never, text ),
  m_isOn( false )
{
}

BitTableItem::~BitTableItem()
{
}

void
BitTableItem::paint( QPainter *p, const QColorGroup & cg, 
                     const QRect & r, bool )
{
  if (isChecked())
  {
    const QBrush fill( cg.highlight() );

    qDrawWinButton( p, 0, 0, r.width(), r.height(), 
                    cg, isChecked(), &fill );
    p->setPen( cg.highlightedText() );
    p->drawText( 0, 0, r.width(), r.height(), AlignCenter, text() );
  }
  else
  {
    const QBrush fill( row() & 1 ? cg.background().dark(110) : cg.background() );

    qDrawWinButton( p, 0, 0, r.width(), r.height(), 
                    cg, isChecked(), &fill );
    p->drawText( 0, 0, r.width(), r.height(), AlignCenter, text() );
  }  
}

QSize
BitTableItem::sizeHint() const
{
  const QFontMetrics fm = table()->fontMetrics();
  
  return QSize( fm.width( text() ) + 8, fm.height()+8 );
}
