//======================================================================
// File:		breadboard.h
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#ifndef BREADBOARD_HH
#define BREADBOARD_HH

#include <qptrlist.h>
#include <qscrollview.h>

#include <breadboardpart.h>

class Pic;
class QPainter;
class MatrixDisplay_7x5;

typedef QPtrList<BreadBoardPart> PartList;
typedef QPtrListIterator<BreadBoardPart> PartListIterator;
 
class BreadBoard : public QScrollView
{
  Q_OBJECT
public:
  enum Mode
  {
    Selection=0,
    CreateConnection
  };
      
  BreadBoard( QWidget *parent=0, const char *name=0 );
  virtual ~BreadBoard();
  
  void setPic( Pic * );
  
  Mode mode() const { return m_mode; }
  
  QPoint gridPos( const QPoint & ) const;
  
public slots:
  void refreshSLOT();
  void show();
  void setModeSLOT( Mode );
  
protected:
  PartList        m_parts;
  BreadBoardPart *m_selectedPart;
  QPoint          m_lastMouse;
  Mode            m_mode;
  bool            m_drawConnection;
  QPoint          m_connectionPos;
  
  void drawContents( QPainter *p, int, int, int, int );
  MatrixDisplay_7x5 *matrixDisplay( Pic * pic );
  
  void contentsMousePressEvent( QMouseEvent *ev );
  void contentsMouseReleaseEvent( QMouseEvent *ev );
  void contentsMouseMoveEvent( QMouseEvent *ev );
  
};

#endif // BREADBOARD_HH
