//======================================================================
// File:		breadboardpart.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <qpainter.h>
#include <qpixmap.h>

#include <picregister.h>
#include <sevensegmentdisplay.h>

#include <center.xpm>
#include <corner.xpm>

QPixmap *BreadBoardPart::s_centerPix = 0;
QPixmap *BreadBoardPart::s_cornerPix = 0;
  
BreadBoardPart::BreadBoardPart( const QString & name ) :
  QObject( 0, name ),
  m_selected( false )
{
  if (!s_cornerPix)
  {
    s_centerPix = new QPixmap((const char **)center_xpm);      
    s_cornerPix = new QPixmap((const char **)corner_xpm);      
  }
  
  m_pinList.setAutoDelete( true );
}

BreadBoardPart::~BreadBoardPart()
{
}

void
BreadBoardPart::moveTo( const QPoint & pos )
{
  m_rect = QRect( pos.x(), pos.y(), 
                  m_rect.width(), m_rect.height() );
}

void
BreadBoardPart::move( int dx, int dy )
{
  m_rect = QRect( m_rect.x()+dx, m_rect.y()+dy, 
                  m_rect.width(), m_rect.height() );
}

void
BreadBoardPart::drawHandles( QPainter *p )
{
  if (selected())
  {
/*    p->drawPixmap( -8, -8, *s_cornerPix );
    p->drawPixmap( -8, rect().height(), *s_cornerPix );
    p->drawPixmap( rect().width(), -8, *s_cornerPix );
    p->drawPixmap( rect().width(), rect().height(), *s_cornerPix );

    p->drawPixmap( (rect().width()>>1)-8, (rect().height()>>1)-8, *s_centerPix  ); */
    p->fillRect( -8, -8, 8, 8, Qt::red );        
    p->fillRect( rect().width(), -8, 8, 8, Qt::red );        
    p->fillRect( -8, rect().height(), 8, 8, Qt::red );        
    p->fillRect( rect().width(), rect().height(), 8, 8, Qt::red );        
  }
}

void
BreadBoardPart::addPin( int no, int x, int y, 
                        BreadBoardPartPin::Orientation orientation,
                        const QString & name )
{
  m_pinList.append( new BreadBoardPartPin( no, x, y, orientation, name ) );
}

void
BreadBoardPart::drawPins( QPainter *p )
{
  p->setPen( Qt::black );
  
  for (PinListIterator it( m_pinList ); *it; ++it)
  {
    (*it)->paint( p );
  }
}

int
BreadBoardPart::hitPin( const QPoint & pos )
{
  for (PinListIterator it( m_pinList ); *it; ++it)
  {
    std::cerr << "    Checking " << (*it)->name().latin1() << std::endl;
    
    if ((*it)->hit( pos - rect().topLeft() ))
    {
      return (*it)->pinNumber();
    }
  }
  
  // no hit
  //
  return -1;
}

