//======================================================================
// File:		breadboardpart.h
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#ifndef BREADBOARDPART_HH
#define BREADBOARDPART_HH

#include <qrect.h>
#include <qobject.h>
#include <qptrlist.h>
#include <breadboardpartpin.h>

class QPainter;
class QPixmap;

class PicRegister;

typedef QPtrList<BreadBoardPartPin> PinList;
typedef QPtrListIterator<BreadBoardPartPin> PinListIterator;

class BreadBoardPart : public QObject
{
  Q_OBJECT
public:
  BreadBoardPart( const QString & );
  virtual ~BreadBoardPart();
    
  QRect rect() const { return m_rect; }
  void moveTo( const QPoint & );
  void move( int, int );

  virtual void paint( QPainter * ) = 0;
  
  bool selected() const { return m_selected; }
  void setSelected( bool sel ) { m_selected = sel; }
    
  void addPin( int, int x, int y, BreadBoardPartPin::Orientation, 
               const QString & );
  
  int hitPin( const QPoint & );
  void drawHandles( QPainter * );
  
protected:
  PinList m_pinList;
  QRect   m_rect;
  bool    m_selected;
  
  static QPixmap *s_cornerPix; 
  static QPixmap *s_centerPix; 
  
  void drawPins( QPainter * );
  
};

#endif // BREADBOARDPART_HH
