//======================================================================
// File:		breadboardpartpin.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <qpixmap.h>
#include <qpainter.h>

#include <iostream>

#include <breadboardpartpin.h>

#include <unused_pin_1.xpm>
#include <unused_pin_2.xpm>
#include <unused_pin_3.xpm>
#include <unused_pin_4.xpm>

#include <used_pin_1.xpm>
#include <used_pin_2.xpm>
#include <used_pin_3.xpm>
#include <used_pin_4.xpm>

#include <input_pin_1.xpm>
#include <input_pin_2.xpm>
#include <input_pin_3.xpm>
#include <input_pin_4.xpm>

#include <output_pin_1.xpm>
#include <output_pin_2.xpm>
#include <output_pin_3.xpm>
#include <output_pin_4.xpm>

#include <stdlib.h>   // random()

QPixmap *BreadBoardPartPin::s_unused[4] = { 0,0,0,0 };
QPixmap *BreadBoardPartPin::s_used[4] = { 0,0,0,0 };
QPixmap *BreadBoardPartPin::s_input[4]  = { 0,0,0,0 };
QPixmap *BreadBoardPartPin::s_output[4] = { 0,0,0,0 };

BreadBoardPartPin::BreadBoardPartPin( int no, int x, int y, 
                                      Orientation orientation,
                                      const QString & name ) :
  m_pos( QPoint( x, y ) ),
  m_orientation( orientation ),
  m_status( (Status)(random() % 4) ),
  //m_status( Unused ),
  m_name( name ),
  m_pinNumber( no )
{
  if (!s_unused[0])
  {
    s_unused[0] = new QPixmap((const char **)unused_pin_1_xpm);
    s_unused[1] = new QPixmap((const char **)unused_pin_2_xpm);
    s_unused[2] = new QPixmap((const char **)unused_pin_3_xpm);
    s_unused[3] = new QPixmap((const char **)unused_pin_4_xpm);
    
    s_used[0] = new QPixmap((const char **)used_pin_1_xpm);
    s_used[1] = new QPixmap((const char **)used_pin_2_xpm);
    s_used[2] = new QPixmap((const char **)used_pin_3_xpm);
    s_used[3] = new QPixmap((const char **)used_pin_4_xpm);
    
    s_input[0] = new QPixmap((const char **)input_pin_1_xpm);
    s_input[1] = new QPixmap((const char **)input_pin_2_xpm);
    s_input[2] = new QPixmap((const char **)input_pin_3_xpm);
    s_input[3] = new QPixmap((const char **)input_pin_4_xpm);

    s_output[0] = new QPixmap((const char **)output_pin_1_xpm);
    s_output[1] = new QPixmap((const char **)output_pin_2_xpm);
    s_output[2] = new QPixmap((const char **)output_pin_3_xpm);
    s_output[3] = new QPixmap((const char **)output_pin_4_xpm);
  }  
}

BreadBoardPartPin::~BreadBoardPartPin()
{
}

void
BreadBoardPartPin::paint( QPainter *p )
{
  switch (m_status)
  {
  case Unused:
    p->drawPixmap( m_pos.x(), m_pos.y(), *s_unused[m_orientation] );
    break;
  case Used:
    p->drawPixmap( m_pos.x(), m_pos.y(), *s_used[m_orientation] );
    break;
  case Input:
    p->drawPixmap( m_pos.x(), m_pos.y(), *s_input[m_orientation] );
    break;
  case Output:
    p->drawPixmap( m_pos.x(), m_pos.y(), *s_output[m_orientation] );
    break;
  }
  
  switch (m_orientation)
  {
  case ConnectRight:
    p->drawText( m_pos.x()+20, m_pos.y(), 200, 16, 
                 Qt::AlignLeft | Qt::AlignVCenter, m_name );
    break;
  case ConnectBottom:
    p->drawText( m_pos.x()-100, m_pos.y(), 200, 16, 
                 Qt::AlignCenter, m_name );
    break;
  case ConnectLeft:
    p->drawText( m_pos.x()-204, m_pos.y(), 200, 16, 
                 Qt::AlignRight | Qt::AlignVCenter, m_name );
    break;
  case ConnectTop:
    p->drawText( m_pos.x()-100, m_pos.y()-16, 200, 16, 
                 Qt::AlignCenter, m_name );
    break;
  }
}

bool
BreadBoardPartPin::hit( const QPoint & pos )
{
  std::cerr << "pin test " << pos.x() << " " << pos.y()
            << " - " << m_pos.x() << " " << m_pos.y() << std::endl;
  
  return ( pos.x() >= m_pos.x()     && 
           pos.x() <  m_pos.x()+16  &&
           pos.y() >= m_pos.y()     && 
           pos.y() <  m_pos.y()+16 );
}

