//======================================================================
// File:		breakcondition.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <breakcondition.h>
#include <picram.h>

#include <iostream>

BreakCondition::BreakCondition( PicRAM * ram ) :
  m_ram( ram ),
  m_reg( 0 ),
  m_compareValue( 0 ),
  m_mode( Always )
{
}

BreakCondition::~BreakCondition()
{
}

bool
BreakCondition::match() const
{
  if (m_mode == Always) return true;
  
  const int value = m_reg->valuePic();
  
  switch (m_mode)
  {
  case GreaterThan:
    return (value > m_compareValue);
  case GreaterOrEqual:
    return (value >= m_compareValue);
  case LessThan:
    return (value < m_compareValue);
  case LessOrEqual:
    return (value <= m_compareValue);
  case Equal:
    return (value == m_compareValue);
  case NotEqual:
    return (value != m_compareValue);
  case And:
    return (value & m_compareValue);
  default:
    return true;    // shut up compiler
  }
  
  return true;
}
 
void
BreakCondition::setParameter( int addr, BreakCondition::Mode mode, int compare )
{
  if (addr != -1)
  {
    m_reg = m_ram->get( addr );
    m_mode = mode;
    m_compareValue = compare;
  }
}
   
