//======================================================================
// File:		breakconditionedit.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <breakconditionedit.h>
#include <picram.h>

BreakConditionEdit::BreakConditionEdit( PicRAM *ram,
                                        QWidget *parent, const char *name ) :
  QLineEdit( parent, name ),
  m_ram( ram ),
  m_mode( BreakCondition::Always )
{
  connect( this, SIGNAL( textChanged( const QString & ) ),
           this, SLOT( textChangedSLOT( const QString & )));
}

BreakConditionEdit::~BreakConditionEdit()
{
}

void
BreakConditionEdit::textChangedSLOT( const QString & text )
{
  QString pattern = text.simplifyWhiteSpace().stripWhiteSpace().upper();
   
  //std::cerr << tmpPattern.latin1() << std::endl;
  
  if (pattern.isEmpty())
  {
    setMode( BreakCondition::Always );
    setOk( true );
  }
  
  QString command;
  QString compare;
  QString regAddr;
  
  unsigned pos = 0;
  
  // get addr of register (if any)
  //
  while (pos < pattern.length() && 
         !isCondition( pattern.constref(pos) ) &&
         !pattern.constref(pos).isSpace())
  {
    regAddr += pattern.constref(pos);
    ++pos;
  }
  
  skipWhiteSpace( &pattern, &pos );
  
  // get condition
  //
  while (pos < pattern.length() && 
         isCondition( pattern.constref(pos) ))
  {
    command += pattern.constref(pos);
    ++pos;
  }
  
  skipWhiteSpace( &pattern, &pos );
  
  // get condition value
  //
  while (pos < pattern.length() && 
         isHexNumber( pattern.constref(pos) ))
  {
    compare += pattern.constref(pos);
    ++pos;
  }
  
  if (regAddr.isEmpty())
  {
    m_registerAddr = -1;
    setOk( false );
    return;
  }
  else
  {
    m_registerAddr = m_ram->symbolToAddr( regAddr );

    if (m_registerAddr == -1)
    {
      bool ok=false;
      m_registerAddr = regAddr.toInt( &ok, 16 ) & 0x7ff ;

      if (!ok && m_registerAddr >= 0 && m_registerAddr <= 0x7ff)
      {
        m_registerAddr = -1;
        setOk( false );
        return;
      }
    }
  }
  
  m_compareValue = -1;
  
  if (!compare.isEmpty())
  {
    bool ok=false;
    m_compareValue = compare.toInt( &ok, 16 ) & 0xff;
     
    if (!ok)  
    { 
      m_compareValue = -1;
    }
  }
  
  if      (command == "==") setMode( BreakCondition::Equal );
  else if (command == "!=") setMode( BreakCondition::NotEqual );
  else if (command == ">")  setMode( BreakCondition::GreaterThan );
  else if (command == ">=") setMode( BreakCondition::GreaterOrEqual );
  else if (command == "<")  setMode( BreakCondition::LessThan );
  else if (command == "<=") setMode( BreakCondition::LessOrEqual );
  else if (command == "&")  setMode( BreakCondition::And );
  else                      setMode( BreakCondition::Always );
  
  setOk( false );
  
  if (m_registerAddr != -1 && m_compareValue != -1)
  {
    setOk( true );
  }
}

void
BreakConditionEdit::setOk( bool ok )
{
  if (ok)
  {
    setBackgroundColor( colorGroup().base() );
  }
  else
  {
    int red   = colorGroup().base().red();
    int green = colorGroup().base().green() - 20;
    int blue  = colorGroup().base().blue() - 20;
    
    if (green < 0) green = 0;
    if (blue < 0)  blue = 0;
    
    setBackgroundColor( QColor( red, green, blue ) );
  }
}
