//======================================================================
// File:		breakpointview.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <qheader.h>
#include <qtable.h>
#include <qtable.h>
#include <qpixmap.h>

#include <pic.h>
#include <breakpointview.h>
#include <labeltableitem.h>
#include <breaktableitem.h>
#include <enabletableitem.h>

#include <bp.xpm>
#include <ibp.xpm>
#include <cbp.xpm>
#include <icbp.xpm>

#include <iostream>

BreakpointView::BreakpointView( QWidget *parent, const char *name ) :
  UIBreakpointView( parent, name ),
  m_pic( 0 )
{
  //m_breakpoints.setAutoDelete( true );
  
  ui_table->setLeftMargin( 0 );
  ui_table->verticalHeader()->hide();
 
  ui_table->setColumnReadOnly( 0, true );
  ui_table->setColumnReadOnly( 4, true );
  ui_table->setColumnReadOnly( 5, true );
  
  QHeader *hheader = ui_table->horizontalHeader();
  hheader->setLabel( 0, "" );
  hheader->resizeSection( 0, 16 );
  
  int hw = QMAX( fontMetrics().width( "000" ), fontMetrics().width( tr("Address") )) + 8;  
  hheader->resizeSection( 1, hw );
  
  hw = fontMetrics().width( "Break Condition" ) + 16;
  hheader->resizeSection( 2, hw );
  
  m_bpPix   = new QPixmap((const char **)bp_xpm);
  m_ibpPix  = new QPixmap((const char **)ibp_xpm);
  m_cbpPix  = new QPixmap((const char **)cbp_xpm);
  m_icbpPix = new QPixmap((const char **)icbp_xpm);

  connect( ui_table, SIGNAL( valueChanged( int, int ) ),
           this, SLOT( valueChangedSLOT( int, int ) )); 
  connect( ui_table, SIGNAL( clicked( int, int, int, const QPoint & ) ),
           this, SLOT( clickedSLOT( int, int, int, const QPoint & ) ));  
}

BreakpointView::~BreakpointView()
{
}

void
BreakpointView::setPic( Pic *pic )
{
  m_pic = pic;
}

void
BreakpointView::setEditable( bool /*editable*/ )
{
  //ui_table->setReadOnly( !editable );
}

void
BreakpointView::newBreakpointSLOT( int addr )
{
  const int row = ui_table->numRows();
  ui_table->setNumRows( row+1 );
  
  QString str;
  str.sprintf( "%03X", addr );
  ui_table->setItem( row, 1, 
        new LabelTableItem( ui_table, str, AlignLeft|AlignVCenter, true ) );
  
  Breakpoint *bp = m_pic->rom()->breakpoint( addr );
  
  ui_table->setItem( row, 2, 
        new BreakTableItem( ui_table, bp->condition() ) );
  EnableTableItem *check = new EnableTableItem( ui_table, "Enabled" );
  check->setChecked( true );
  ui_table->setItem( row, 3, check );
  
  setPixmap( bp, row );
}

void
BreakpointView::removeBreakpointSLOT( int addr )
{
  int row = -1;
  
  for (int i=0; i<ui_table->numRows(); ++i)
  {
    int tmpAddr = ui_table->text( i, 1 ).toInt( 0, 16 );
    
    if (tmpAddr == addr)
    {
      row = i;
      break;
    }
  }
  
  if (row != -1)
  {
    ui_table->removeRow( row );
  }
}

void
BreakpointView::changedBreakpointSLOT( int addr )
{
  int row = -1;
  
  for (int i=0; i<ui_table->numRows(); ++i)
  {
    int tmpAddr = ui_table->text( i, 1 ).toInt( 0, 16 );
    
    if (tmpAddr == addr)
    {
      row = i;
      break;
    }
  }
  
  if (row != -1)
  {
    Breakpoint *bp = m_pic->rom()->breakpoint( addr );
    
    EnableTableItem *check = dynamic_cast<EnableTableItem *> (ui_table->item( row, 3 ));
      
    if (check)
    {
      check->setChecked( bp->isEnabled() );
    }
    
    setPixmap( bp, row );
  }
}

void
BreakpointView::clickedSLOT( int row, int col, int button, const QPoint &  )
{
  int addr = ui_table->text( row, 1 ).toInt( 0, 16 );

  Breakpoint *bp = m_pic->rom()->breakpoint( addr );
  
  if (bp)
  {
    if (col == 3 && button == LeftButton)
    {
      EnableTableItem *check = dynamic_cast<EnableTableItem *> (ui_table->item( row, col ));

      if (check)
      {
        check->setChecked( !check->isChecked() );

        bp->setEnabled( check->isChecked() );
      }
    }

    setPixmap( bp, row );
    
    emit changedBreakpoint( addr );
  }
}

void
BreakpointView::valueChangedSLOT( int row, int /*col*/ )
{
  int addr = ui_table->text( row, 1 ).toInt( 0, 16 );

  Breakpoint *bp = m_pic->rom()->breakpoint( addr );
  
  setPixmap( bp, row );
  
  emit changedBreakpoint( addr );
}

void
BreakpointView::setPixmap( Breakpoint *bp, int row )
{
  if (bp->isEnabled())
  {
    if (bp->isConditional())
    {
      ui_table->setPixmap( row, 0, *m_cbpPix );
    }
    else
    {
      ui_table->setPixmap( row, 0, *m_bpPix );
    }
  }
  else
  {
    if (bp->isConditional())
    {
      ui_table->setPixmap( row, 0, *m_icbpPix );
    }
    else
    {
      ui_table->setPixmap( row, 0, *m_ibpPix );
    }
  }
}

