//======================================================================
// File:		breaktableitem.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <qpainter.h>
#include <qlineedit.h>
#include <qvalidator.h>
#include <qregexp.h>

#include <breaktableitem.h>
#include <picregister.h>
#include <breakcondition.h>
#include <breakconditionedit.h>

#include <iostream>

BreakTableItem::BreakTableItem( QTable *table, BreakCondition *cond ) :
  QTableItem( table, QTableItem::OnTyping ),
  m_condition( cond )
{
}

BreakTableItem::~BreakTableItem()
{
}

QWidget  *
BreakTableItem::createEditor() const
{
  BreakConditionEdit *edit = new BreakConditionEdit( m_condition->ram(),
                                                     table()->viewport() );
  edit->setText( text() );
  
  return edit;
}

void 
BreakTableItem::setContentFromEditor( QWidget * w )
{
  BreakConditionEdit *edit = dynamic_cast<BreakConditionEdit *> (w);
  
  if (edit)
  {
    setText( edit->text().upper() );
    
    m_condition->setParameter( edit->address(), edit->mode(), edit->value() );
  }
}
 
void 
BreakTableItem::paint( QPainter *p, const QColorGroup & cg, 
                        const QRect & rect, bool )
{
  const QRect cr( 0, 0, rect.width(), rect.height() );

  p->fillRect( cr, row() & 1 ? cg.base().dark( 105 ) : cg.base() );
  
  p->drawText( 4, 0, cr.width()-8, cr.height(), 
               AlignVCenter | AlignLeft, text() );
}
    
