//======================================================================
// File:		clockview.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <qlabel.h>
#include <qpushbutton.h>

#include <clockview.h>
#include <pic.h>
#include <stringconversion.h>

ClockView::ClockView( QWidget *parent, const char *name ) :
  UIClockView( parent, name ),
  m_pic( 0 )
{
  connect( ui_clear, SIGNAL( clicked() ),
           this, SLOT( clearSLOT() ));
  
  startTimer( 100 );
}

ClockView::~ClockView()
{
}

void
ClockView::setPic( Pic *pic )
{
  m_pic = pic;
  
  refreshSLOT();
}

void 
ClockView::timerEvent( QTimerEvent * )
{
  refreshSLOT();
}

void
ClockView::refreshSLOT()
{
  static QString str;
  
  if (!m_pic) return;
 
  ui_crystal->setText( StringConversion::freqString( m_pic->crystalFrequency() ) );
  
  str.sprintf( "%u", m_pic->cycles() );
  ui_cycles->setText( str );
  
  str.sprintf( "%.2fs", m_pic->usElapsed() / 1e6 );
  ui_time->setText( str );
}

void
ClockView::clearSLOT()
{
  if (!m_pic) return;
  std::cerr << "ClockView::clearSLOT()" << std::endl;
  m_pic->clearCycles();
}
