//======================================================================
// File:		cod.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <qfile.h>
#include <qdatastream.h>

#include <pic.h>
#include <cod.h>

#include <iostream>
#include <stdio.h>
#include <string.h>

CodBlock::CodBlock( char *data )
{
  m_data = new unsigned char [BlockSize];
  memcpy( m_data, data, BlockSize );
}

CodBlock::~CodBlock()
{
  delete [] m_data; 
}

DirBlock::DirBlock( char *data ) :
    CodBlock( data )
{
}

DirBlock::~DirBlock()
{
}

DataBlock::DataBlock( char *data ) :
    CodBlock( data )
{
}

DataBlock::~DataBlock()
{
}

NameBlock::NameBlock( char *data ) :
    CodBlock( data )
{
}

NameBlock::~NameBlock()
{
}

SymbolBlock::SymbolBlock( char *data ) :
    CodBlock( data )
{
}

SymbolBlock::~SymbolBlock()
{
}

int
SymbolBlock::numSymbols() const
{
  unsigned char *ptr = m_data;
  int len=0;
  int numSymbols = 0;
  
  while (len<CodBlock::BlockSize)
  {
    int off = ptr[SymbolLength] + Offset;
    
    if (0 == ptr[SymbolLength]) return numSymbols;
    
    ptr += off;
    len += off;
    numSymbols++;
    
/*    std::cerr << "off=" << off << " len=" << len
              << " num=" << numSymbols << std::endl; */
  }  
  
  return 0;
}

int
SymbolBlock::symbolOffset( int i ) const
{
  //int tmpi = i;
  unsigned char *data = m_data;
  int len = 0;
  
  while (i)
  {
    int off = data[SymbolLength] + Offset;
    data += off;
    len += off;
    --i;
    
    if (len >= CodBlock::BlockSize) return -1;
  }
  
  //std::cerr << "offset " << tmpi << " = " << len << std::endl;
  return len;
}

QString
SymbolBlock::symbol( int i ) const
{
  int off = symbolOffset(i);
  
  if (off == -1) return QString::null;
  
  unsigned char *data = m_data+off;

  QString str;
  
  for (int i=0; i<data[SymbolLength]; ++i)
  {
    str += data[i+1];
  }
  
  return str;
}

int
SymbolBlock::symbolType( int i ) const
{
  int off = symbolOffset(i);
  
  if (off == -1) return -1;
  
  unsigned char *data = m_data+off;

  data += data[SymbolLength] + 1;
  
  return *data;
}

int
SymbolBlock::symbolValue( int i ) const
{
  int off = symbolOffset(i);
  
  if (off == -1) return -1;
  
  unsigned char *data = m_data+off;

  off += data[SymbolLength];
  
  // constants are 8 bit
  //
  /*if (m_data[off+1] == 47)
  {
    return (unsigned)byte( off+6 );
  }*/
  
  return value( off+5 );
}

Cod::Cod( const QString &filename, Pic *pic ) :
  m_dirBlock( 0 ),
  m_numBlocks( 0 )
{
  if (readFile( filename ))
  {
    SymbolTable *symTab = pic->symbolTable();
    
    symTab->clear();
        
    std::cerr << "  CODE:       " << m_dirBlock->code() << std::endl;
    std::cerr << "  FILE:       " << m_dirBlock->filename() << std::endl;
    std::cerr << "  SOURCE:     " << m_dirBlock->source() << std::endl;
    std::cerr << "  COPY:       " << m_dirBlock->copyright() << std::endl;
    std::cerr << "  PROCESSOR:  " << m_dirBlock->processor() << std::endl;
    std::cerr << "  DATE:       " << m_dirBlock->date() << std::endl;
    std::cerr << "  TIME:       " << m_dirBlock->time() << std::endl;
    std::cerr << "  VERSION:    " << m_dirBlock->version() << std::endl;
    std::cerr << "  COMPILER:   " << m_dirBlock->compiler() << std::endl;
    std::cerr << "  MEMORY MAP: " << m_dirBlock->memoryMap() << std::endl;
    std::cerr << "  ADDRESSSIZE:" << m_dirBlock->addressSize() << std::endl;
    std::cerr << "  NEXT DIR:   " << m_dirBlock->nextDir() << std::endl;
    std::cerr << "  COD TYPE:   " << m_dirBlock->codType() << std::endl;
    std::cerr << "  LONG SYMB:  " << m_dirBlock->longSymbols() << std::endl;
    std::cerr << "  DEBUG:      " << m_dirBlock->debugMessages() << std::endl;
    std::cerr << "  SYBOLS:     " << m_dirBlock->symbolTable() << std::endl;
    std::cerr << "  NAMES:      " << m_dirBlock->nameTable() << std::endl;
    
    if (m_dirBlock->nameTable())
    {
      int off = m_dirBlock->nameTable()-1;
      
      NameBlock *block = (NameBlock *) (m_block[off]);

      for (int i=0; i<NameBlock::NumNames; ++i)
      {
        std::cerr << "NAME: " << block->name(i) << std::endl;
      }
    }
    
    if (m_dirBlock->longSymbols())
    {
      int off = m_dirBlock->longSymbols()-1;
      
      int numBlocks = m_dirBlock->nameTable()-m_dirBlock->longSymbols();
      
      for (int b=0; b<numBlocks; ++b)
      {
        SymbolBlock *block = (SymbolBlock *) (m_block[off+b]);

        //std::cerr << "num symbols: " << block->numSymbols() << std::endl;

        int numSymbols = block->numSymbols();
        for (int i=0; i<numSymbols; ++i)
        {
          symTab->addSymbol( new Symbol( block->symbol(i), 
                                         block->symbolValue(i),
                                         block->symbolType(i) == 47 ? 
                                            Symbol::Constant : Symbol::Address ) );
        }        
      }
      
      // iterate through symbols and set names of register
      // 
      pic->ram()->clearNames();
      SymbolIterator constIt = pic->symbolTable()->constantsIterator();

      for ( ; constIt.current(); ++constIt)
      {
        QString str;

        // retrieve register (if any)
        // later we can GUESS if the constant is meant to be a register
        //
        PicRegister *reg = 0;

        if (constIt.current()->value() <= 0x1ff)
        {
          reg = pic->ram()->get( constIt.current()->value() );
        }

        // GUESS if it's a general purpose register
        //
        if (reg && reg->type() == PicRegister::GeneralPurpose)
        {
          reg->setName( constIt.current()->name() );
        }
      } 
    }
    
    if (m_dirBlock->code())
    {
      unsigned data[2048];
      
      memset( data, 0, sizeof(unsigned) * 2048 );
      
      // This is rather weird stuff
      // first data block is at codeblock position
      // remaining blocks appear to reside after 
      // memory map block
      //
      
      int off = m_dirBlock->code()-1;
      
      DataBlock *block = (DataBlock *) (m_block[off]);
      
      for (int i=0; i<256; ++i)
      {
        data[i] = block->shortInt(2*i);
      }
      
      off = m_dirBlock->memoryMap();
      int index = 256;
      
      for (int b=off; b<m_numBlocks; ++b)
      {
        block = (DataBlock *) (m_block[b]);
        
        for (int i=0; i<256; ++i)
        {
          data[index++] = block->shortInt(2*i);
        }
      }
      
      pic->loadROM( data, 2048 );
    }
  }
}

Cod::~Cod()
{
  delete m_dirBlock;
  
  for (int i=0; i<m_numBlocks; ++i)
  {
    delete m_block[i];
  }
}

bool
Cod::readFile( const QString & filename )
{
  QFile file( filename );
  if (file.open(IO_ReadOnly))
  {
    QDataStream s( &file );
    char *data = new char [CodBlock::BlockSize];
    
    m_numBlocks = 0;
    
    s.readRawBytes( data, CodBlock::BlockSize );
    m_dirBlock = new DirBlock( data );  
      
    while (!s.atEnd())
    {
      s.readRawBytes( data, CodBlock::BlockSize );
      m_block[m_numBlocks++] = new DataBlock( data );  
    }
    
    delete [] data;
    
    file.close();
    
    std::cerr << "successfully loaded (" << m_numBlocks << ")" << std::endl;
    
    return true;
  }

  std::cerr << "file open fuckup " << filename.latin1() << std::endl;
  
  return false;
}
