//======================================================================
// File:		disass.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <disass.h>

DisAss::DisAss() :
  m_count( 0 )
{
}

DisAss::~DisAss()
{
}

void
DisAss::setData( unsigned *data, int count )
{
  m_data = data;
  m_count = count;
}

QString 
DisAss::disassemble()
{
  QString ret;
  
  for (int i=0; i<m_count; ++i)
  {
    QString tmp;
    tmp.sprintf( "[0x%04x] : 0x%04x - ", i, m_data[i] & 0x3fff );
    
    unsigned bit = 0x2000;
    
    for (int b=0; b<14; ++b)
    {
      if (m_data[i] & bit)
      {
        tmp += "1";
      }
      else
      {
        tmp += "0";
      }
      
      if (b==1 || b==5 || b==9)
      {
        tmp += " ";
      }
      
      bit >>= 1;
    }
    tmp += "\t";
    
    ret += tmp;
    
    ret += DisAss::toString( m_data[i] );
    ret += "\n";
  }
  
  return ret;
}

QString
DisAss::toString( unsigned opcode )
{
  QString ret;
  
  // special opcodes
  //
  if (opcode == 0x0008)
  {
    ret += "return";
    return ret;
  }
  else if (opcode == 0x0063)
  {
    ret += "sleep";
    return ret;
  }
  else if (opcode == 0x0009)
  {
    ret += "retfie";
    return ret;
  }
  else if (opcode == 0x0064)
  {
    ret += "clrwdt";
    return ret;
  }
  else if (opcode == 0x0062)
  {
    ret += "option";
    return ret;
  }
  else if ((opcode & 0x3ff8) == 0x60)
  {
    ret += "tris    ";
    ret += decodeTRISF( opcode );
  }
  
  // byte oriented stuff
  //
  const unsigned op = (opcode >> 8) & 0x003f;
  
  if ((op & 0x0030) == 0)
  {
    if (op == 0x0007)
    {
      ret += "addwf  ";
      ret += decodeFD( opcode );
      return ret;
    }
    else if (op == 0x0005)
    {
      ret += "andwf  ";
      ret += decodeFD( opcode );
      return ret;
    }
    else if (op == 0x0001)
    {
      if (opcode & 0x0080)
      {
        ret += "clrf   ";
        ret += decodeF( opcode );
        return ret;
      }
      else
      {
        ret += "clrw   ";
        return ret;
      }
    }
    else if (op == 0x0009)
    {
      ret += "comf   ";
      ret += decodeFD( opcode );
      return ret;
    }
    else if (op == 0x0003)
    {
      ret += "decf   ";
      ret += decodeFD( opcode );
      return ret;
    }
    else if (op == 0x000b)
    {
      ret += "decfsz ";
      ret += decodeFD( opcode );
      return ret;
    }
    else if (op == 0x000a)
    {
      ret += "incf   ";
      ret += decodeFD( opcode );
      return ret;
    }
    else if (op == 0x000f)
    {
      ret += "incfsz ";
      ret += decodeFD( opcode );
      return ret;
    }
    else if (op == 0x0004)
    {
      ret += "iorwf  ";
      ret += decodeFD( opcode );
      return ret;
    }
    else if (op == 0x0008)
    {
      ret += "movf   ";
      ret += decodeFD( opcode );
      return ret;
    }
    else if (op == 0x0000)
    {
      if (opcode & 0x0080)
      {
        ret += "movwf  ";
        ret += decodeF( opcode );
        return ret;
      }
      else
      {
        ret += "nop";
        return ret;
      }
    }
    else if (op == 0x000d)
    {
      ret += "rlf    ";
      ret += decodeFD( opcode );
      return ret;
    }
    else if (op == 0x000c)
    {
      ret += "rrf    ";
      ret += decodeFD( opcode );
      return ret;
    }
    else if (op == 0x0002)
    {
      ret += "subwf  ";
      ret += decodeFD( opcode );
      return ret;
    }
    else if (op == 0x000e)
    {
      ret += "swapf  ";
      ret += decodeFD( opcode );
      return ret;
    }
    else if (op == 0x0006)
    {
      ret += "xorwf  ";
      ret += decodeFD( opcode );
      return ret;
    }
  }
  
  // bit oriented stuff
  //
  if ((op & 0x0030) == 0x0010)
  {
    const unsigned op2 = (opcode >> 10) & 0x000f;
    
    if (op2 == 0x0004)
    {
      ret += "bcf    ";
      ret += decodeFB( opcode );
      return ret;
    }
    
    if (op2 == 0x0005)
    {
      ret += "bsf    ";
      ret += decodeFB( opcode );
      return ret;
    }
    
    if (op2 == 0x0006)
    {
      ret += "btfsc  ";
      ret += decodeFB( opcode );
      return ret;
    }
    
    if (op2 == 0x0007)
    {
      ret += "btfss  ";
      ret += decodeFB( opcode );
      return ret;
    }
  }
  
  // literal stuff
  //
  if ((op & 0x0030) == 0x0030)
  {
    if ((op & 0x003e) == 0x003e)    
    {
      ret += "addlw  ";
      ret += decodeL( opcode );
      return ret;
    }
    
    if (op == 0x0039)    
    {
      ret += "andlw  ";
      ret += decodeL( opcode );
      return ret;
    }
    
    if (op == 0x0038)    
    {
      ret += "iorlw  ";
      ret += decodeL( opcode );
      return ret;
    }

    if ((op & 0x003c) == 0x0030)    
    {
      ret += "movlw  ";
      ret += decodeL( opcode );
      return ret;
    }

    if ((op & 0x003c) == 0x0034)    
    {
      ret += "retlw  ";
      ret += decodeL( opcode );
      return ret;
    }

    if ((op & 0x003e) == 0x003c)    
    {
      ret += "sublw  ";
      ret += decodeL( opcode );
      return ret;
    }

    if (op == 0x003a)    
    {
      ret += "xorlw  ";
      ret += decodeL( opcode );
      return ret;
    }
  }
   
  // jump stuff
  //
  if ((op & 0x0030) == 0x0020)
  {
    if ((op & 0x0038) == 0x0020)    
    {
      ret += "call   ";
      ret += decodeAddr( opcode );
      return ret;
    }
  
    if ((op & 0x0038) == 0x0028)    
    {
      ret += "goto   ";
      ret += decodeAddr( opcode );
      return ret;
    }
  }
    
  if (opcode == 0)
  {
    ret += "nop";
  }
  else
  {
    ret += "illegal";
  }
  
  return ret;
}

QString
DisAss::decodeF( unsigned opcode )
{
  QString ret;
  ret.sprintf( "0x%02X", opcode & 0x007f );
  
  return ret;
}

QString
DisAss::decodeTRISF( unsigned opcode )
{
  QString ret;
  ret.sprintf( "%d", opcode & 0x0007 );
  
  return ret;
}

QString
DisAss::decodeFD( unsigned opcode )
{
  QString ret;
  ret.sprintf( "0x%02X, %c", opcode & 0x007f, (opcode & 0x0080 ? 'F' : 'W') );
  
  return ret;
}

QString
DisAss::decodeFB( unsigned opcode )
{
  QString ret;
  ret.sprintf( "0x%02X, %d", opcode & 0x007f, ((opcode >> 7) & 0x07) );
  
  return ret;
}

QString
DisAss::decodeL( unsigned opcode )
{
  QString ret;
  ret.sprintf( "0x%02X (%d)", (opcode & 0x00ff), (opcode & 0x00ff) );
  
  return ret;
}

QString
DisAss::decodeAddr( unsigned opcode )
{
  QString ret;
  ret.sprintf( "0x%03X", (opcode & 0x07ff) );
  
  return ret;
}
