//======================================================================
// File:		editor.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <qapplication.h>
#include <qfile.h>
#include <qtextedit.h>
#include <qtextstream.h>
#include <qprocess.h>
#include <qlistbox.h>
#include <qdir.h>
#include <qfiledialog.h>
#include <qtoolbutton.h>
#include <qlabel.h>
#include <qbuttongroup.h>
#include <qlineedit.h>

#include <textedit.h>
#include <pic.h>
#include <editor.h>
#include <picsyntaxhighlighter.h>
#include <replacedlg.h>
#include <finddlg.h>
#include <gotodlg.h>
#include <outputitem.h>
#include <cod.h>
#include <stringconversion.h>

#include <iostream>

Editor::Editor( QWidget *parent, const char *name ) :
  UIEditor( parent, name ),
  m_findPara( 0 ),
  m_findIndex( 0 ),
  m_findDlg( 0 ),
  m_replaceDlg( 0 ),
  m_gotoDlg( 0 ),
  m_currentFilename( "noname" ),
  m_fileDialog( 0 ),
  m_makeWasOk( false )
{
  new PicSyntaxHighlighter( ui_editor );
  
  m_asm = new QProcess( this );
  
  connect( m_asm, SIGNAL( readyReadStdout() ),
           this, SLOT( readStdoutSLOT() ));
  connect( m_asm, SIGNAL( readyReadStderr() ),
           this, SLOT( readStderrSLOT() ));
  connect( m_asm, SIGNAL( processExited() ),
           this, SLOT( processExitedSLOT() ));
  
  connect( ui_output, SIGNAL( doubleClicked( QListBoxItem * ) ),
           this, SLOT( outputDoubleClickedSLOT( QListBoxItem * ) ));
  
  ui_editor->resetPcToLineTable();
  
  connect( ui_editor, SIGNAL( newBreakpointRequest( int ) ),
           this, SIGNAL( newBreakpointRequest( int ) ));
  connect( ui_editor, SIGNAL( removeBreakpointRequest( int ) ),
           this, SIGNAL( removeBreakpointRequest( int ) ));
  connect( ui_editor, SIGNAL( editBreakpointRequest( int ) ),
           this, SIGNAL( editBreakpointRequest( int ) ));
  connect( ui_editor, SIGNAL( runHereRequest( int ) ),
           this, SIGNAL( runHereRequest( int ) ));
  connect( ui_editor, SIGNAL( addToWatchRequest( PicRegister * ) ),
           this, SIGNAL( addToWatchRequest( PicRegister * ) ));
  connect( ui_editor, SIGNAL( removeFromWatchRequest( PicRegister * ) ),
           this, SIGNAL( removeFromWatchRequest( PicRegister * ) ));
  connect( ui_editor, SIGNAL( resetRequest() ),
           this, SIGNAL( resetRequest() ));
  connect( ui_editor, SIGNAL( runRequest() ),
           this, SIGNAL( runRequest() ));
  connect( ui_editor, SIGNAL( finishCallRequest() ),
           this, SIGNAL( finishCallRequest() ));
  connect( ui_editor, SIGNAL( animateRequest() ),
           this, SIGNAL( animateRequest() ));
  connect( ui_editor, SIGNAL( stepRequest() ),
           this, SIGNAL( stepRequest() ));
  connect( ui_editor, SIGNAL( stepOverRequest() ),
           this, SIGNAL( stepOverRequest() ));
  connect( ui_editor, SIGNAL( stopRequest() ),
           this, SIGNAL( stopRequest() ));
  connect( ui_editor, SIGNAL( changedBreakpoint( int ) ),
           this, SIGNAL( changedBreakpoint( int ) ));
  
  connect( ui_editor, SIGNAL( textChanged() ),
           this, SLOT( textChangedSLOT() ));
  
  connect( ui_statusGroup, SIGNAL( clicked( int ) ),
           this, SLOT( statusGroupSLOT( int ) ));
}

Editor::~Editor()
{
}

void
Editor::textChangedSLOT()
{
  m_pic->setDirty( ui_editor->isModified() || !m_makeWasOk );
  createCaption();
  
  if (m_pic->dirty())
  {
    ui_editor->resetPcToLineTable();
  }
}

void
Editor::updateBreakpoints()
{
  ui_editor->update();
}

QString
Editor::text() const
{
  return ui_editor->text();
}

void
Editor::showFindSLOT()
{
  if (!m_findDlg)
  {
    m_findDlg = new FindDlg( this );
    
    connect( m_findDlg, SIGNAL( find() ),
             this, SLOT( findSLOT() ));
  }
  
  m_findDlg->show();
}

void
Editor::showReplaceSLOT()
{
  if (!m_replaceDlg)
  {
    m_replaceDlg = new ReplaceDlg( this );
    
    connect( m_replaceDlg, SIGNAL( replace() ),
             this, SLOT( replaceSLOT() ));
  }
  
  m_replaceDlg->show();
}

void
Editor::findSLOT()
{
  if (!m_findDlg) return;
  
  if (!m_findDlg->keepDialog())
  {
    m_findDlg->hide();
  }
  
  find( m_findDlg->text(), m_findDlg->caseSensitive(), 
        m_findDlg->wordsOnly(), m_findDlg->forward() );
}

void
Editor::findAgainSLOT()
{
  if (!m_findDlg) return;
  
  find( m_findDlg->text(), m_findDlg->caseSensitive(), 
        m_findDlg->wordsOnly(), m_findDlg->forward() );
}

void
Editor::replaceSLOT()
{
}

void
Editor::replaceAgainSLOT()
{
}

void
Editor::gotoLineSLOT()
{
  if (!m_gotoDlg)
  {
    m_gotoDlg = new GotoDlg( this );
  }
  
  if (m_gotoDlg->exec())
  {
    if (m_gotoDlg->ok())
    {
      gotoLine( m_gotoDlg->lineNumber() );
    }
  }
}

void
Editor::gotoPcSLOT( int pc )
{
  gotoLine( ui_editor->lineForPc( pc ) );
}

void
Editor::cutSLOT()
{
  ui_editor->cut();
}

void
Editor::copySLOT()
{
  ui_editor->copy();
}

void
Editor::pasteSLOT()
{
  ui_editor->paste();
}

void
Editor::undoSLOT()
{
  ui_editor->undo();
}

void
Editor::redoSLOT()
{
  ui_editor->redo();
}

void
Editor::selectAllSLOT()
{
  ui_editor->selectAll();
}

void
Editor::upperSLOT()
{
  QString text = ui_editor->selectedText().upper();
  ui_editor->insert( text );
}

void
Editor::lowerSLOT()
{
  QString text = ui_editor->selectedText().lower();
  ui_editor->insert( text );
}

void 
Editor::find( const QString & expr, bool cs, bool wo, bool forward )
{
  int selParaFrom, selIndexFrom, selParaTo, selIndexTo;
  
  ui_editor->getSelection( &selParaFrom, &selIndexFrom,
                           &selParaTo, &selIndexTo );
  
  ui_editor->getCursorPosition( &m_findPara, &m_findIndex );
  
  if (m_findPara == selParaFrom && m_findIndex == selIndexFrom)
  {
    if (forward)
    {
      m_findPara = selParaTo;
      m_findIndex = selIndexTo;
    }
  }
  
  if (!ui_editor->find( expr, cs, wo, forward, &m_findPara, &m_findIndex ))
  {
    if (forward)
    {
      m_findPara = 0;
      m_findIndex = 0;
    }
    else
    {
      m_findPara = ui_editor->paragraphs()-1;
      m_findIndex = ui_editor->text( m_findPara ).length()-1;
    }
    
    if (!ui_editor->find( expr, cs, wo, forward, &m_findPara, &m_findIndex ))
    {
      QString msg;
      msg.sprintf( tr("Couldn't find %s"), expr.latin1() );
      emit message( msg );
      qApp->beep();
    }
  }
}

void
Editor::gotoLine( int lineNumber )
{
  int len = ui_editor->text( lineNumber ).length();
  
  ui_editor->setSelection( lineNumber, 0, lineNumber, len );
  ui_editor->setFocus();
}

void
Editor::gotoSelectionSLOT()
{
  ui_editor->ensureCursorVisible();
}

void
Editor::makeSLOT()
{
  if (ui_editor->isModified())
  {
    emit message( tr( "Source was modified: Please save." ) );
    qApp->beep();
    return;
  }
  
  QStringList arg;
  arg.append( "gpasm" );
  arg.append( "--force-list" );
  arg.append( m_currentFilename );
  
  m_asm->setArguments( arg );
  
  ui_output->clear();
  
  QString line;
  for (unsigned i=0; i<arg.count(); ++i)
  {
    line += arg[i];
    line += " ";
  }
  
  new OutputItem( line, OutputItem::Message, ui_output );
  
  m_errorsOrWarnings = false;
  m_makeWasOk = false;
  
  if (!m_asm->start())
  {
    line.prepend( tr("Couldn't execute: '") );
    line += "'";
    
    emit message( line );
    qApp->beep();
  }
  else
  {
    qApp->setOverrideCursor( waitCursor );
  }
}

void
Editor::readStdoutSLOT()
{
  QString line = m_asm->readLineStdout();
  
  if (line.contains( "Warning" ))
  {
    m_errorsOrWarnings = true;
    new OutputItem( line, OutputItem::Warning, ui_output );
  }
  else if (line.contains( "Error"))
  {
    m_errorsOrWarnings = true;
    new OutputItem( line, OutputItem::Error, ui_output );
  }
  else
  {
    new OutputItem( line, OutputItem::Message, ui_output );
  }
}

void 
Editor::readStderrSLOT()
{
  new OutputItem( m_asm->readLineStderr(), OutputItem::Error, ui_output );
}

void 
Editor::processExitedSLOT()
{
  while (m_asm->canReadLineStdout())
  {
    readStdoutSLOT();
  }
  while (m_asm->canReadLineStderr())
  {
    readStderrSLOT();
  }
  
  if (m_asm->normalExit())
  {
    new OutputItem( tr("Normal exit"), OutputItem::Message, ui_output );
    
    parseLST();
    m_pic->setDirty( false );
    m_makeWasOk = true;
    
    QString msg;
    
    if (m_errorsOrWarnings)
    {
      msg = tr("gpasm found errors or warnings");
    }
    else
    {
      msg = tr("gpasm finished without errors or warnings");
    }
    emit message( msg );  
  }
  else
  {
    QString msg;
    msg.sprintf( tr("Command returned exit code %d"), m_asm->exitStatus() );
    new OutputItem( msg, OutputItem::Error, ui_output );
    
    emit message( msg );
    
    m_pic->setDirty( true );
  }
  
  qApp->restoreOverrideCursor();
}

void
Editor::saveSLOT()
{
  QFile file( m_currentFilename );
  
  if (file.open( IO_WriteOnly ))
  {
    QTextStream ts(&file);
    ts << ui_editor->text();
    file.close();
    
    ui_editor->setModified( false );
    createCaption();
    
    QString msg;
    msg.sprintf( tr("File %s saved"), m_currentFilename.latin1() );
    emit message( msg );
  }
}

void
Editor::openSLOT()
{
  if (!m_fileDialog)
  {
    m_fileDialog = new QFileDialog( this, "opendlg", true );
    m_fileDialog->addFilter( tr("PIC Assembler (*.asm *.inc)") );
    m_fileDialog->setDir( QDir::currentDirPath() );
    m_fileDialog->setSelectedFilter( 1 );
    m_fileDialog->setCaption( tr("Open file - YaPIDE") );
    m_fileDialog->setMode( QFileDialog::ExistingFile );
  }
  
  if (m_fileDialog->exec())
  {
    open( m_fileDialog->selectedFile() );
  }
}

void
Editor::open( const QString & filename )
{
  m_currentFilename = filename;
  
  createCaption();
  
  QFile file( filename );
  
  if (file.open( IO_ReadOnly ))
  {
    QTextStream ts(&file);
    QString text;
    text = ts.read();
    file.close();
    
    ui_editor->resetPcToLineTable();
    ui_editor->setText( text );  
    
    m_pic->setDirty( true );
    
    QString msg;
    msg.sprintf( tr("File %s opened"), m_currentFilename.latin1() );
    emit message( msg );
  }
}

void
Editor::createCaption()
{
  QString cap = "Editor - ";
  cap += m_currentFilename;
  if (ui_editor->isModified())
  {
    cap += tr( " (modified)" );
  }
  setCaption( cap );
}

void
Editor::outputDoubleClickedSLOT( QListBoxItem *item )
{
  QStringList list = QStringList::split( ":", item->text() );
  
  if (list.count() > 2 && 
      (list[2].startsWith( "Error" ) ||
       list[2].startsWith( "Message" ) ||
       list[2].startsWith( "Warning" )))
  {
    gotoLine( list[1].toInt( 0, 10 )-1 );
  }
}

void
Editor::parseLST()
{
  ui_editor->resetPcToLineTable();
    
  // TODO: Filename handling is still crappy. We assume special filenames
  //       
  QString baseName = m_currentFilename;
  baseName.truncate( baseName.length()-3 );
  
  QString lstName = baseName + "lst";
  
  QFile file( lstName );
  
  if (file.open( IO_ReadOnly ))
  {
    QTextStream ts( &file );
    
    QString line;
    
    while (!ts.atEnd())
    {
      line = ts.readLine();  
      if (line.length() >= 20)
      {
        QString addr = line.left( 4 ).lower();
        QString lineNo = line.mid( 15, 5 );
        
        bool addrOk = true;
        
        for (int i=0; i<4; ++i)
        {
          if (!addr[i].isNumber() &&
              addr[i] != 'a' &&
              addr[i] != 'b' &&
              addr[i] != 'c' &&
              addr[i] != 'd' &&
              addr[i] != 'e' &&
              addr[i] != 'f')
          {
            addrOk = false;
            break;
          }
        }
        
        bool lineNoOk = true;
        
        for (int i=0; i<5; ++i)
        {
          if (!lineNo[i].isNumber())
          {
            lineNoOk = false;
            break;
          }
        }
        
        if (addrOk && lineNoOk)
        {
          int iAddr = addr.toInt( 0, 16);
          int iLine = lineNo.toInt( 0, 10 )-1;
          ui_editor->setLine( iAddr, iLine );
        }
      }
    }
  }
    
  QString codName = baseName + "cod";
  Cod( codName, m_pic );
  
  m_pic->reset();
  
  emit reloadPic();
  
  refreshSLOT();
}

void
Editor::refreshSLOT()
{
  ui_editor->setPc( m_pic->pc() );
  ui_editor->update();
  
  int status = m_pic->sfr( Pic::STATUS )->valuePic();
  
  ui_c->setOn( status & Pic::C_BIT );
  ui_dc->setOn( status & Pic::DC_BIT );
  ui_z->setOn( status & Pic::Z_BIT );
  ui_pd->setOn( status & Pic::nPD_BIT );
  ui_t0->setOn( status & Pic::nTO_BIT );
  ui_rp0->setOn( status & Pic::RP0_BIT );
  ui_rp1->setOn( status & Pic::RP1_BIT );
  ui_irp->setOn( status & Pic::IRP_BIT );
  
  ui_wHex->setText( StringConversion::hexString( m_pic->workingRegister() ));
  ui_wDec->setText( StringConversion::decString( m_pic->workingRegister() ));
  ui_wBin->setText( StringConversion::binString( m_pic->workingRegister() ));
}

void
Editor::setPic( Pic *pic )
{
  m_pic = pic;
  ui_editor->setPic( pic );
}

void
Editor::statusGroupSLOT( int id )
{
  if (ui_statusGroup->find( id )->isOn())
  {
    m_pic->sfr( Pic::STATUS )->setBitPic( id );
  }
  else
  {
    m_pic->sfr( Pic::STATUS )->clearBitPic( id );
  }
  
  m_pic->sfr( Pic::STATUS )->freeze();
  
  emit updateViews();
}
