//======================================================================
// File:		editor.h
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#ifndef EDITOR_HH
#define EDITOR_HH

#include <uieditor.h>

class ReplaceDlg;
class FindDlg;
class GotoDlg;
class QProcess;
class QFileDialog;
class Pic;
class PicRegister;

class Editor : public UIEditor
{
  Q_OBJECT
public:
  Editor( QWidget *parent=0, const char *name=0 );
  virtual ~Editor();

  QString text() const;
  
  void setPic( Pic * );
  void updateBreakpoints();
  
signals:
  void reloadPic();
  void newBreakpointRequest( int );
  void removeBreakpointRequest( int );
  void editBreakpointRequest( int );
  void runHereRequest( int );
  void addToWatchRequest( PicRegister * );
  void removeFromWatchRequest( PicRegister * );
  void resetRequest();
  void runRequest();
  void finishCallRequest();
  void animateRequest();
  void stepRequest();
  void stepOverRequest();
  void stopRequest();
  void message( const QString & );
  void updateViews();
  
public slots:
  void cutSLOT();
  void copySLOT();
  void pasteSLOT();
  void undoSLOT();
  void redoSLOT();
  void saveSLOT();
  void openSLOT();
  void selectAllSLOT();
  void gotoSelectionSLOT();
  void findSLOT();
  void replaceSLOT();
  void showFindSLOT();
  void showReplaceSLOT();
  void findAgainSLOT();
  void replaceAgainSLOT();
  void gotoLineSLOT();
  void gotoPcSLOT( int );
  void makeSLOT();
  void upperSLOT();
  void lowerSLOT();
  void refreshSLOT();
  
protected:
  int          m_findPara;
  int          m_findIndex;
  FindDlg     *m_findDlg;
  ReplaceDlg  *m_replaceDlg;
  GotoDlg     *m_gotoDlg;
  QString      m_currentFilename;
  QProcess    *m_asm;
  QFileDialog *m_fileDialog;
  int          m_pcToLine[2048];
  Pic         *m_pic;
  bool         m_makeWasOk;
  bool         m_errorsOrWarnings;
  
  virtual void find( const QString & expr, bool cs, bool wo, bool forward );
  void gotoLine( int );
  void open( const QString & );
  void createCaption();
  void parseLST();
  
protected slots:
  void readStdoutSLOT();
  void readStderrSLOT();
  void processExitedSLOT();
  void outputDoubleClickedSLOT( QListBoxItem *item );
  void textChangedSLOT();
  void statusGroupSLOT( int id );
  
};

#endif // EDITOR_HH
