//======================================================================
// File:		indfregister.h
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#ifndef INDFREGISTER_HH
#define INDFREGISTER_HH

#include <picram.h>
#include <picregister.h>

class INDFRegister : public PicRegister
{
public:
  INDFRegister( PicRAM * );
  ~INDFRegister();
  
  int valuePic();
  bool bitSetPic( int bit ) const;
  bool bitSet( int bit ) const;
  void setBit( int bit );
  void clearBit( int bit );
  void setBitPic( int bit );
  void clearBitPic( int bit );
  int valuePic() const;
  void setValuePic( int value );
  void setValue( int value );
  int value() const;
  
protected:
  PicRAM *m_ram;
  
  PicRegister *address() const;
  
};

inline PicRegister *
INDFRegister::address() const
{
  int addr = m_ram->fsr()->valuePic();
  
  if (m_ram->status()->bitSetPic( 7 ))
  {
    addr |= 0x100;
  }
  
  if (0 == addr) return m_ram->null();
  
  return m_ram->get(addr);
}

inline int
INDFRegister::valuePic()
{
  return address()->valuePic();
}

inline bool
INDFRegister::bitSetPic( int bit ) const
{
  return address()->bitSetPic( bit );
}

inline bool
INDFRegister::bitSet( int bit ) const
{
  return address()->bitSet( bit );
}
 
inline void
INDFRegister::setBit( int bit )
{
  address()->setBit( bit );
}

inline void
INDFRegister::clearBit( int bit )
{
  address()->clearBit( bit );
}

inline void
INDFRegister::setBitPic( int bit )
{
  address()->setBitPic( bit );
}

inline void
INDFRegister::clearBitPic( int bit )
{
  address()->clearBitPic( bit );
}

inline int
INDFRegister::valuePic() const
{
  return address()->valuePic();
}

inline void
INDFRegister::setValuePic( int value )
{
  address()->setValuePic( value );
}

inline void
INDFRegister::setValue( int value )
{
  address()->setValue( value );
}

inline int
INDFRegister::value() const
{
  return address()->value();
}

#endif // INDFREGISTER_HH
