//======================================================================
// File:		instructiontracebuffer.h
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#ifndef INSTRUCTIONTRACEBUFFER_HH
#define INSTRUCTIONTRACEBUFFER_HH

#include <qglobal.h>
#include <tracebuffer.h>

class InstructionTraceBuffer : public TraceBuffer
{
public:
  InstructionTraceBuffer();
  ~InstructionTraceBuffer();

  void add( Q_UINT32 pc, Q_UINT32 w, Q_UINT32 status );
  void add( Q_UINT32 pc, Q_UINT32 w, Q_UINT32 status, Q_UINT32 file );
    
  int pc( int pos ) const;
  int w( int pos ) const;
  int status( int pos ) const;
  bool hasFile( int pos ) const;
  int file( int pos ) const;
  
protected:
  Q_UINT32 *m_buffer;
  
};

inline void
InstructionTraceBuffer::add( Q_UINT32 pc, Q_UINT32 w, Q_UINT32 status )
{
  m_buffer[m_curPointer++] = pc | 
                             (w << 12) | 
                             ((status & 0x07) << 20);
  m_curPointer &= 0xfff;
  
  if (m_length < 0xfff) m_length++;
}

inline void
InstructionTraceBuffer::add( Q_UINT32 pc, Q_UINT32 w, 
                             Q_UINT32 status, Q_UINT32 file )
{
  m_buffer[m_curPointer++] = pc | 
                             (w << 12) | 
                             ((status & 0x07) << 20) | 
                             (file << 24) | 
                             0x00800000;
  m_curPointer &= 0xfff;
  
  if (m_length < 0xfff) m_length++;
}

inline int
InstructionTraceBuffer::pc( int pos ) const
{
  return m_buffer[offset(pos)] & 0x7ff;
}

inline int
InstructionTraceBuffer::w( int pos ) const
{
  return (m_buffer[offset(pos)] >> 12 ) & 0xff;
}

inline int
InstructionTraceBuffer::status( int pos ) const
{
  return (m_buffer[offset(pos)] >> 20 ) & 0x07;
}

inline bool
InstructionTraceBuffer::hasFile( int pos ) const
{
  return (m_buffer[offset(pos)] & 0x00800000);
}

inline int
InstructionTraceBuffer::file( int pos ) const
{
  return (m_buffer[offset(pos)] >> 24 ) & 0xff;
}

#endif // INSTRUCTIONTRACEBUFFER_HH
