//======================================================================
// File:		lowpassdouble.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <lowpassdouble.h>

#include <iostream>

LowPassDouble::LowPassDouble() :
  m_tap( 0 ),
  m_numTaps( 22 )//10 )
{
  m_taps = new double [m_numTaps];
  m_coef = new double [m_numTaps];
  
  clearTaps();
  /*
  m_coef[0] = 1.0/5.0;
  m_coef[1] = 1.0/5.0;
  m_coef[2] = 1.0/5.0;
  m_coef[3] = 1.0/5.0;
  m_coef[4] = 1.0/5.0;
  */

#if 1
  for (int i=0; i<m_numTaps; ++i)
  {
    m_coef[i] = 1.0 / (double)m_numTaps;
  }
#else
  // LOWPASS
  // passband upper: 0.1
  // stopband lower: 0.2
  // passband ripple: 2
  // stopband attenuation: 50
  //
/*  m_coef[0] = 0.014931;
  m_coef[1] = 0.064156;
  m_coef[2] = 0.132373;
  m_coef[3] = 0.210227;
  m_coef[4] = 0.259681;
  m_coef[5] = 0.259681;
  m_coef[6] = 0.210227;
  m_coef[7] = 0.132373;
  m_coef[8] = 0.064156;
  m_coef[9] = 0.014931;
  */
      
  m_coef[0]  = 0.002874079027;
  m_coef[1]  = 0.005780056618;
  m_coef[2]  = 0.004303352395;
  m_coef[5]  = -0.007351954765;
  m_coef[6]  = -0.028353270070;
  m_coef[7]  = -0.045803301348;
  m_coef[8]  = -0.038000131360;
  m_coef[9]  = 0.011968675934;
  m_coef[10] = 0.197719998626;
  m_coef[11] = 0.261767930515;
  m_coef[12] = 0.261767930515;
  m_coef[13] = 0.197719998626;
  m_coef[14] = 0.011968675934;
  m_coef[15] = -0.038000131360;
  m_coef[16] = -0.045803301348;
  m_coef[17] = -0.028353270070;
  m_coef[18] = -0.007351954765;
  m_coef[19] = 0.004303352395;
  m_coef[20] = 0.005780056618;
  m_coef[21] = 0.002874079027;
      
  double sum = 0.0;
  
  for (int i=0; i<22; ++i)
  {
    sum += m_coef[i];
  }
  
  std::cerr << "sum=" << sum << std::endl;
  
  for (int i=0; i<10; ++i)
  {
    m_coef[i] /= sum;
  }

#endif
}

LowPassDouble::LowPassDouble( int numTaps ) :
  m_tap( 0 ),
  m_numTaps( numTaps )
{
  m_taps = new double [m_numTaps];
  
  clearTaps();
}

void
LowPassDouble::clearTaps()
{
  for (int i=0; i<m_numTaps; ++i)
  {
    m_taps[i] = 0.0;
  }
}

LowPassDouble::~LowPassDouble()
{
  delete [] m_taps;
}

void
LowPassDouble::setSample( double sample )
{
  m_taps[m_tap] = sample;
  
  m_value = 0.0;
  
  for (int i=0; i<m_numTaps; ++i)
  {
    m_value += m_taps[(i+m_tap) % m_numTaps] * m_coef[i];
  }
  
  ++m_tap;
  if (m_tap >= m_numTaps)  m_tap = 0;
}

