//======================================================================
// File:		mainwindow.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <qaction.h>
#include <qapplication.h>
#include <qmenubar.h>
#include <qpixmap.h>
#include <qpopupmenu.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qwhatsthis.h>
#include <qlabel.h>

#include <mainwindow.h>
#include <workspace.h>
#include <stringconversion.h>

#include <uiabout.h>

#include <find_file.xpm>
#include <about.xpm>
#include <animation.xpm>
#include <breakpoint.xpm>
#include <brownout.xpm>
#include <clock.xpm>
#include <close.xpm>
#include <copy.xpm>
#include <cpu.xpm>
#include <cut.xpm>
#include <direct_help.xpm>
#include <editor.xpm>
#include <make.xpm>
#include <new.xpm>
#include <open.xpm>
#include <paste.xpm>
#include <qt.xpm>
#include <quit.xpm>
#include <ram.xpm>
#include <redo.xpm>
#include <reset.xpm>
#include <rom.xpm>
#include <run.xpm>
#include <save.xpm>
#include <save_as.xpm>
#include <search.xpm>
#include <stack.xpm>
#include <stop.xpm>
#include <stop_after_command.xpm>
#include <stop_after_procedure.xpm>
#include <symbols.xpm>
#include <trace.xpm>
#include <undo.xpm>
#include <watch.xpm>
#include <yapide_logo.xpm>

MainWindow::MainWindow( QWidget *parent, const char *name ) :
  QMainWindow( parent, name )
{
  setCaption( tr("Main Window - YaPIDE") );
  
  m_workspace = new Workspace( this );
  setCentralWidget( m_workspace );
  
  connect( m_workspace, SIGNAL( updateMenus( bool ) ),
           this, SLOT( updateMenusSLOT( bool ) ));
  connect( m_workspace, SIGNAL( message( const QString & ) ),
           statusBar(), SLOT( message( const QString & ) ));
  
  createActions();
  createMenu();
  createToolBars();
  
  setIcon(QPixmap((const char **)yapide_logo_xpm));
  
  m_frequencyTimer = startTimer( 1000 );
  
  statusBar();
  
  m_status = new QLabel( statusBar() );
  statusBar()->addWidget( m_status, 0, true );
  
  setUsesBigPixmaps( true );
  //setUsesTextLabel( true );
}

MainWindow::~MainWindow()
{
}

void
MainWindow::timerEvent( QTimerEvent *ev )
{
  if (ev->timerId() == m_frequencyTimer)
  {
    //m_workspace->freeze();
    //m_workspace->refreshSLOT();
     
    if (m_workspace->halted())
    {
      //statusBar()->message( tr("Halted") );
      m_status->setText( tr("Halted") );
    }
    else
    {
      QString str;
      str.sprintf( "16F628 %s - Running @ %s",
          StringConversion::freqString_g( m_workspace->crystalFrequency() ).latin1(), 
          StringConversion::freqString( m_workspace->clockSpeed() ).latin1() );

      //statusBar()->message( str );
      m_status->setText( str );
    }
  }
}

void
MainWindow::createActions()
{
//  QIconSet::setIconSize( QIconSet::Large, QSize( 24, 24 ) );
//  QIconSet::setIconSize( QIconSet::Small, QSize( 16, 16 ) );
  
  m_quitAction = new QAction( tr("Quit"), QIconSet(QPixmap((const char**)quit_xpm)),//, QIconSet::Large), 
                              tr("Quit"), QKeySequence( CTRL+Key_Q ),
                              this );
  m_showRAMAction = new QAction( tr("RAM"), QIconSet(QPixmap((const char**)ram_xpm)),//, QIconSet::Large),  
                                 tr("RAM"), QKeySequence(),
                                 this, 0, true );
  m_showROMAction = new QAction( tr("ROM"), QIconSet(QPixmap((const char**)rom_xpm)),//, QIconSet::Large), 
                                 tr("ROM"), QKeySequence(),
                                 this, 0, true );
  m_showCPUAction = new QAction( tr("CPU"), QIconSet(QPixmap((const char**)cpu_xpm)),//, QIconSet::Large),  
                                 tr("CPU"), QKeySequence(),
                                 this, 0, true );
  m_showStackAction = new QAction( tr("Stack"), QIconSet(QPixmap((const char**)stack_xpm)),//, QIconSet::Large),  
                                   tr("Stack"), QKeySequence(),
                                   this, 0, true );
  m_showClockAction = new QAction( tr("Clock"), QIconSet(QPixmap((const char**)clock_xpm), QIconSet::Large), 
                                   tr("Clock"), QKeySequence(),
                                   this, 0, true );
  m_showWatchAction = new QAction( tr("Watch"), QIconSet(QPixmap((const char**)watch_xpm), QIconSet::Large),  
                                   tr("Watch"), QKeySequence(),
                                   this, 0, true );
  m_showEditorAction = new QAction( tr("Editor"), QIconSet(QPixmap((const char**)editor_xpm), QIconSet::Large),  
                                    tr("Editor/Debugger"), QKeySequence(),
                                    this, 0, true );
  m_showBreakpointsAction = new QAction( tr("Breakpoints"), QIconSet(QPixmap((const char**)breakpoint_xpm), QIconSet::Large),  
                                         tr("Breakpoints"), QKeySequence(),
                                         this, 0, true );
  m_showSymbolsAction = new QAction( tr("Symbols"), QIconSet(QPixmap((const char**)symbols_xpm), QIconSet::Large),  
                                         tr("Symbols"), QKeySequence(),
                                         this, 0, true );
  m_showInstructionTraceAction = new QAction( tr("Instruction Trace"), QIconSet(QPixmap((const char**)trace_xpm), QIconSet::Large),  
                                         tr("Instruction Trace"), QKeySequence(),
                                         this, 0, true );
  
  m_freeRunAction = new QAction( tr("Free Run"), tr("Free Run"), QKeySequence(), this, 0, true );

  m_tracingAction = new QAction( tr("Tracing"), tr("Tracing"), QKeySequence(), this, 0, true );

  m_resetPicAction = new QAction( tr("Reset"), QIconSet(QPixmap((const char**)reset_xpm), QIconSet::Large),  
                                  tr("Reset"), 
                                  QKeySequence(), this );
  m_brownoutPicAction = new QAction( tr("Brownout"), QIconSet(QPixmap((const char**)brownout_xpm), QIconSet::Large),  
                                     tr("Brownout"), 
                                    QKeySequence(), this );
  m_runPicAction = new QAction( tr("Run"), QIconSet(QPixmap((const char**)run_xpm), QIconSet::Large),  
                                tr("Run"), 
                                QKeySequence(), this );
  m_stepPicAction = new QAction( tr("Step"), QIconSet(QPixmap((const char**)stop_after_command_xpm), QIconSet::Large),  
                                 tr("Step"), 
                                 QKeySequence( Key_F2 ), this );
  m_overPicAction = new QAction( tr("Step Over"), QIconSet(QPixmap((const char**)stop_after_procedure_xpm), QIconSet::Large),  
                                 tr("Step Over"), 
                                 QKeySequence( Key_F3 ), this );
  m_finishCallPicAction = new QAction( tr("Finish CALL"),  
                                 tr("Finish CALL"), 
                                 QKeySequence( Key_F4 ), this );
  m_stopPicAction = new QAction( tr("Stop"), QIconSet(QPixmap((const char**)stop_xpm), QIconSet::Large),  
                                 tr("Stop"), 
                                 QKeySequence( Key_F5 ), this );
  m_animatePicAction = new QAction( tr("Animate"), QIconSet(QPixmap((const char**)animation_xpm), QIconSet::Large),  
                                    tr("Animate"), 
                                    QKeySequence(), this );

  m_aboutAction = new QAction( tr("About YaPIDE"), QIconSet(QPixmap((const char**)about_xpm), QIconSet::Large),  
                               tr("About YaPIDE ..."), 
                               QKeySequence(), this );
  m_aboutQtAction = new QAction( tr("About Qt"), QIconSet(QPixmap((const char**)qt_xpm), QIconSet::Large), 
                                 tr("About Qt ..."), 
                                 QKeySequence(), this );
  m_whatsThisAction = new QAction( tr("What's this"), QIconSet(QPixmap((const char**)direct_help_xpm), QIconSet::Large), 
                                   tr("What's this ..."), 
                                   QKeySequence(), this );
  
  connect( m_quitAction, SIGNAL( activated() ),
           this, SLOT( quitSLOT() ));
  connect( m_whatsThisAction, SIGNAL( activated() ),
           this, SLOT( enterWhatsThisModeSLOT() ));
  
  connect( m_showRAMAction, SIGNAL( toggled( bool ) ),
           m_workspace, SLOT( showRAMSLOT( bool ) ));
  connect( m_showROMAction, SIGNAL( toggled( bool ) ),
           m_workspace, SLOT( showROMSLOT( bool ) ));
  connect( m_showCPUAction, SIGNAL( toggled( bool ) ),
           m_workspace, SLOT( showCPUSLOT( bool ) ));
  connect( m_showStackAction, SIGNAL( toggled( bool ) ),
           m_workspace, SLOT( showStackSLOT( bool ) ));
  connect( m_showClockAction, SIGNAL( toggled( bool ) ),
           m_workspace, SLOT( showClockSLOT( bool ) ));
  connect( m_showWatchAction, SIGNAL( toggled( bool ) ),
           m_workspace, SLOT( showWatchSLOT( bool ) ));
  connect( m_showEditorAction, SIGNAL( toggled( bool ) ),
           m_workspace, SLOT( showEditorSLOT( bool ) ));
  connect( m_showBreakpointsAction, SIGNAL( toggled( bool ) ),
           m_workspace, SLOT( showBreakpointsSLOT( bool ) ));
  connect( m_showSymbolsAction, SIGNAL( toggled( bool ) ),
           m_workspace, SLOT( showSymbolsSLOT( bool ) ));
  connect( m_showInstructionTraceAction, SIGNAL( toggled( bool ) ),
           m_workspace, SLOT( showInstructionTraceSLOT( bool ) ));
  
  connect( m_freeRunAction, SIGNAL( toggled( bool ) ),
           m_workspace, SLOT( freeRunSLOT( bool ) ));
  connect( m_tracingAction, SIGNAL( toggled( bool ) ),
           m_workspace, SLOT( tracingSLOT( bool ) ));
  
  connect( m_resetPicAction, SIGNAL( activated() ),
           m_workspace, SLOT( resetPicSLOT() ));
  connect( m_runPicAction, SIGNAL( activated() ),
           m_workspace, SLOT( runPicSLOT() ));
  connect( m_stepPicAction, SIGNAL( activated() ),
           m_workspace, SLOT( stepPicSLOT() ));
  connect( m_overPicAction, SIGNAL( activated() ),
           m_workspace, SLOT( overPicSLOT() ));
  connect( m_finishCallPicAction, SIGNAL( activated() ),
           m_workspace, SLOT( finishCallPicSLOT() ));
  connect( m_stopPicAction, SIGNAL( activated() ),
           m_workspace, SLOT( stopPicSLOT() ));
  connect( m_animatePicAction, SIGNAL( activated() ),
           m_workspace, SLOT( animatePicSLOT() ));
  
  connect( m_aboutAction, SIGNAL( activated() ),
           this, SLOT( aboutSLOT() ));
  connect( m_aboutQtAction, SIGNAL( activated() ),
           this, SLOT( aboutQtSLOT() ));
  
  m_newFileAction = new QAction( tr("New"), QIconSet(QPixmap((const char**)new_xpm), QIconSet::Large),  
                                 tr("New"), 
                                 QKeySequence( CTRL+Key_N ), this );
  m_openFileAction = new QAction( tr("Open"), QIconSet(QPixmap((const char**)open_xpm), QIconSet::Large),  
                                  tr("Open ..."), 
                                  QKeySequence( CTRL+Key_O ), this );
  m_saveFileAction = new QAction( tr("Save"), QIconSet(QPixmap((const char**)save_xpm), QIconSet::Large),  
                                  tr("Save"), 
                                  QKeySequence( CTRL+Key_S ), this );
  m_saveFileAsAction = new QAction( tr("Save As"), QIconSet(QPixmap((const char**)save_as_xpm), QIconSet::Large),  
                                    tr("Save As ..."), 
                                    QKeySequence(), this );
  m_closeFileAction = new QAction( tr("Close"), QIconSet(QPixmap((const char**)close_xpm), QIconSet::Large),  
                                   tr("Close"), 
                                   QKeySequence(), this );
  m_cutAction = new QAction( tr("Cut"), QIconSet(QPixmap((const char**)cut_xpm), QIconSet::Large),  
                             tr("Cut"), 
                             QKeySequence( CTRL+Key_X ), this );
  m_copyAction = new QAction( tr("Copy"), QIconSet(QPixmap((const char**)copy_xpm), QIconSet::Large),  
                              tr("Copy"), 
                              QKeySequence( CTRL+Key_C ), this );
  m_pasteAction = new QAction( tr("Paste"), QIconSet(QPixmap((const char**)paste_xpm), QIconSet::Large), 
                               tr("Paste"), 
                               QKeySequence( CTRL+Key_V ), this );
  m_makeAction = new QAction( tr("Assemble"), QIconSet(QPixmap((const char**)make_xpm), QIconSet::Large),  
                               tr("Assemble"), 
                               QKeySequence( Key_F9 ), this );
  m_selectAllAction = new QAction( tr("Select ALl"), tr("Select All"), 
                              QKeySequence( CTRL+Key_A ), this );

  m_undoAction = new QAction( tr("Undo"), QIconSet(QPixmap((const char**)undo_xpm), QIconSet::Large),  
                              tr("Undo"), 
                              QKeySequence( CTRL+Key_Z ), this );
  m_redoAction = new QAction( tr("Redo"), QIconSet(QPixmap((const char**)redo_xpm), QIconSet::Large),  
                              tr("Redo"), 
                              QKeySequence( SHIFT+CTRL+Key_Z ), this );
  m_findFileAction = new QAction( tr("Find File"), QIconSet(QPixmap((const char**)find_file_xpm), QIconSet::Large),  
                                  tr("Find File"), 
                                  QKeySequence(), this );
  m_findAction = new QAction( tr("Find"), QIconSet(QPixmap((const char**)search_xpm), QIconSet::Large),  
                              tr("Find ..."), 
                              QKeySequence( CTRL+Key_F ), this );
  m_findAgainAction = new QAction( tr("Find Again"), tr("Find Again"), 
                              QKeySequence( CTRL+Key_G ), this );
  m_replaceAction = new QAction( tr("Replace"), 
                              tr("Replace ..."), 
                              QKeySequence( CTRL+Key_R ), this );
  m_replaceAgainAction = new QAction( tr("Replace Again"), tr("Replace Again"), 
                              QKeySequence( CTRL+Key_T ), this );
  m_gotoLineAction = new QAction( tr("Goto Line"), tr("Goto Line ..."), 
                              QKeySequence( CTRL+Key_L ), this );
  m_gotoSelectionAction = new QAction( tr("Goto Selection"), tr("Goto Selection"), 
                              QKeySequence( CTRL+Key_E ), this );
  m_upperAction = new QAction( tr("Upper-case"), tr("Upper-case"), 
                              QKeySequence( CTRL+Key_6 ), this );
  m_lowerAction = new QAction( tr("Lower-case"), tr("Lower-case"), 
                              QKeySequence( SHIFT+CTRL+Key_6 ), this );
  
  connect( m_saveFileAction, SIGNAL( activated() ),
           m_workspace, SLOT( saveSLOT() ));
  connect( m_openFileAction, SIGNAL( activated() ),
           m_workspace, SLOT( openSLOT() ));
  connect( m_cutAction, SIGNAL( activated() ),
           m_workspace, SLOT( cutSLOT() ));
  connect( m_copyAction, SIGNAL( activated() ),
           m_workspace, SLOT( copySLOT() ));
  connect( m_pasteAction, SIGNAL( activated() ),
           m_workspace, SLOT( pasteSLOT() ));
  connect( m_makeAction, SIGNAL( activated() ),
           m_workspace, SLOT( makeSLOT() ));
  connect( m_selectAllAction, SIGNAL( activated() ),
           m_workspace, SLOT( selectAllSLOT() ));
  connect( m_undoAction, SIGNAL( activated() ),
           m_workspace, SLOT( undoSLOT() ));
  connect( m_redoAction, SIGNAL( activated() ),
           m_workspace, SLOT( redoSLOT() ));
  connect( m_findAction, SIGNAL( activated() ),
           m_workspace, SLOT( showFindSLOT() ));
  connect( m_findAgainAction, SIGNAL( activated() ),
           m_workspace, SLOT( findAgainSLOT() ));
  connect( m_replaceAction, SIGNAL( activated() ),
           m_workspace, SLOT( showReplaceSLOT() ));
  connect( m_replaceAgainAction, SIGNAL( activated() ),
           m_workspace, SLOT( replaceAgainSLOT() ));
  connect( m_gotoLineAction, SIGNAL( activated() ),
           m_workspace, SLOT( gotoLineSLOT() ));
  connect( m_gotoSelectionAction, SIGNAL( activated() ),
           m_workspace, SLOT( gotoSelectionSLOT() ));
  connect( m_upperAction, SIGNAL( activated() ),
           m_workspace, SLOT( upperSLOT() ));
  connect( m_lowerAction, SIGNAL( activated() ),
           m_workspace, SLOT( lowerSLOT() ));
  
  updateMenusSLOT( false );
}

void
MainWindow::createMenu()
{
  QMenuBar *menu = menuBar();
  
  QPopupMenu *file = new QPopupMenu( menu );
  m_newFileAction->addTo( file );
  m_openFileAction->addTo( file );
  file->insertSeparator();
  m_saveFileAction->addTo( file );
  m_saveFileAsAction->addTo( file );
  m_closeFileAction->addTo( file );
  file->insertSeparator();
  m_quitAction->addTo( file );
  
  menu->insertItem( tr("&File"), file );
  
  QPopupMenu *edit = new QPopupMenu( menu );
  //edit->insertTearOffHandle();
  m_undoAction->addTo( edit );
  m_redoAction->addTo( edit );
  edit->insertSeparator();
  m_cutAction->addTo( edit );
  m_copyAction->addTo( edit );
  m_pasteAction->addTo( edit );
  edit->insertSeparator();
  m_selectAllAction->addTo( edit );
  edit->insertSeparator();
  m_upperAction->addTo( edit );
  m_lowerAction->addTo( edit );
  edit->insertSeparator();
  m_makeAction->addTo( edit );
  
  menu->insertItem( tr("&Edit"), edit );
  
  QPopupMenu *search = new QPopupMenu( menu );
  //search->insertTearOffHandle();
  //m_findFileAction->addTo( search );
  m_findAction->addTo( search );
  m_findAgainAction->addTo( search );
  m_replaceAction->addTo( search );
  m_replaceAgainAction->addTo( search );
  search->insertSeparator();
  m_gotoLineAction->addTo( search );
  m_gotoSelectionAction->addTo( search );
  
  menu->insertItem( tr("&Search"), search );
    
  QPopupMenu *simulator = new QPopupMenu( menu );
  //simulator->insertTearOffHandle();
  m_freeRunAction->addTo( simulator );
  m_tracingAction->addTo( simulator );
  simulator->insertSeparator();
  m_resetPicAction->addTo( simulator );
  m_brownoutPicAction->addTo( simulator );
  m_runPicAction->addTo( simulator );
  m_animatePicAction->addTo( simulator );
  m_stepPicAction->addTo( simulator );
  m_overPicAction->addTo( simulator );
  m_finishCallPicAction->addTo( simulator );
  m_stopPicAction->addTo( simulator );
  
  menu->insertItem( tr("Sim&ulator"), simulator );
  
  QPopupMenu *window = new QPopupMenu( menu );
  //window->insertTearOffHandle();
  m_showEditorAction->addTo( window );
  m_showRAMAction->addTo( window );
  m_showROMAction->addTo( window );
  m_showCPUAction->addTo( window );
  m_showStackAction->addTo( window );
  m_showClockAction->addTo( window );
  m_showWatchAction->addTo( window );
  m_showBreakpointsAction->addTo( window );
  m_showSymbolsAction->addTo( window );
  m_showInstructionTraceAction->addTo( window );
  
  menu->insertItem( tr("&Window"), window );
  
  QPopupMenu *help = new QPopupMenu( menu );
  //help->insertTearOffHandle();
  m_aboutAction->addTo( help );
  m_aboutQtAction->addTo( help );
  help->insertSeparator();
  m_whatsThisAction->addTo( help );
  
  menu->insertSeparator();
  menu->insertItem( tr("&Help"), help );
}

void
MainWindow::createToolBars()
{
  QToolBar *file = new QToolBar( this );
  file->setLabel( tr("File operations" ) );
  m_newFileAction->addTo( file );
  m_openFileAction->addTo( file );
  file->addSeparator();
  m_saveFileAction->addTo( file );
  m_saveFileAsAction->addTo( file );
  m_closeFileAction->addTo( file );
  file->addSeparator();
  m_quitAction->addTo( file );
  
  addDockWindow( file );
  
  QToolBar *edit = new QToolBar( this );
  edit->setLabel( tr("Edit operations" ) );
  m_undoAction->addTo( edit );
  m_redoAction->addTo( edit );
  edit->addSeparator();
  m_cutAction->addTo( edit );
  m_copyAction->addTo( edit );
  m_pasteAction->addTo( edit );
  edit->addSeparator();
  m_makeAction->addTo( edit );
  
  addDockWindow( edit );
  
  QToolBar *search = new QToolBar( this );
  search->setLabel( tr("Search operations" ) );
//  m_findFileAction->addTo( search );
  m_findAction->addTo( search );
  
  addDockWindow( search );

  QToolBar *window = new QToolBar( this );
  window->setLabel( tr("Windows" ) );
  m_showEditorAction->addTo( window );
  m_showRAMAction->addTo( window );
  m_showROMAction->addTo( window );
  m_showCPUAction->addTo( window );
  m_showStackAction->addTo( window );
  m_showClockAction->addTo( window );
  m_showWatchAction->addTo( window );
  m_showBreakpointsAction->addTo( window );
  m_showSymbolsAction->addTo( window );
  m_showInstructionTraceAction->addTo( window );
  
  addDockWindow( window );
  
  QToolBar *simulator = new QToolBar( this );
  simulator->setLabel( tr("Simulator" ) );
  m_resetPicAction->addTo( simulator );
  m_brownoutPicAction->addTo( simulator );
  m_runPicAction->addTo( simulator );
  m_animatePicAction->addTo( simulator );
  m_stepPicAction->addTo( simulator );
  m_overPicAction->addTo( simulator );
  m_stopPicAction->addTo( simulator );
  
  addDockWindow( simulator );
  
  QToolBar *help = new QToolBar( this );
  help->setLabel( tr("Help" ) );
  m_whatsThisAction->addTo( help );
  
  addDockWindow( help );
  
  
}

void
MainWindow::updateMenusSLOT( bool run )
{
  if (run)
  {
    m_runPicAction->setEnabled( false );
    m_animatePicAction->setEnabled( false );
    m_stepPicAction->setEnabled( false );
    m_overPicAction->setEnabled( false );
    m_finishCallPicAction->setEnabled( false );
    m_stopPicAction->setEnabled( true );
    
    m_newFileAction->setEnabled( false );
    m_openFileAction->setEnabled( false );
    m_saveFileAction->setEnabled( false );
    m_saveFileAsAction->setEnabled( false );
    m_closeFileAction->setEnabled( false );
    m_makeAction->setEnabled( false );
    
    m_undoAction->setEnabled( false );
    m_redoAction->setEnabled( false );
    m_cutAction->setEnabled( false );
    m_copyAction->setEnabled( false );
    m_pasteAction->setEnabled( false );
    m_selectAllAction->setEnabled( false );
    m_upperAction->setEnabled( false );
    m_lowerAction->setEnabled( false );
  }
  else if (m_workspace->simulatorAnimation())
  {
    m_runPicAction->setEnabled( false );
    m_animatePicAction->setEnabled( false );
    m_stepPicAction->setEnabled( false );
    m_finishCallPicAction->setEnabled( false );
    m_overPicAction->setEnabled( false );
    m_stopPicAction->setEnabled( true );
    
    m_newFileAction->setEnabled( false );
    m_openFileAction->setEnabled( false );
    m_saveFileAction->setEnabled( false );
    m_saveFileAsAction->setEnabled( false );
    m_closeFileAction->setEnabled( false );
    m_makeAction->setEnabled( false );
    
    m_undoAction->setEnabled( false );
    m_redoAction->setEnabled( false );
    m_cutAction->setEnabled( false );
    m_copyAction->setEnabled( false );
    m_pasteAction->setEnabled( false );
    m_selectAllAction->setEnabled( false );
    m_upperAction->setEnabled( false );
    m_lowerAction->setEnabled( false );
  }
  else
  {
    m_animatePicAction->setEnabled( true );
    m_runPicAction->setEnabled( true );
    m_stepPicAction->setEnabled( true );
    if (m_workspace->pic()->insideCall())
    {
      m_finishCallPicAction->setEnabled( true );
    }
    else
    {
      m_finishCallPicAction->setEnabled( false );
    }
    m_overPicAction->setEnabled( true );
    m_stopPicAction->setEnabled( false );
    
    m_newFileAction->setEnabled( true );
    m_openFileAction->setEnabled( true );
    m_saveFileAction->setEnabled( true );
    m_saveFileAsAction->setEnabled( true );
    m_closeFileAction->setEnabled( true );
    m_makeAction->setEnabled( true );
    
    m_undoAction->setEnabled( true );
    m_redoAction->setEnabled( true );
    m_cutAction->setEnabled( true );
    m_copyAction->setEnabled( true );
    m_pasteAction->setEnabled( true );
    m_selectAllAction->setEnabled( true );
    m_upperAction->setEnabled( true );
    m_lowerAction->setEnabled( true );
  }
}

void
MainWindow::quitSLOT()
{
  m_workspace->quit();
  qApp->quit();
}

void
MainWindow::aboutSLOT()
{
  static UIAbout *about=0;
  
  if (!about)
  {
    about = new UIAbout( this, 0, true );
  }
  
  about->exec();
}

void
MainWindow::aboutQtSLOT()
{
  QMessageBox::aboutQt( this, tr("About Qt") );
}

void
MainWindow::enterWhatsThisModeSLOT()
{
  QWhatsThis::enterWhatsThisMode();
}

