//======================================================================
// File:		matrixdisplay_7x5.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <qpainter.h>

#include <picregister.h>
#include <matrixdisplay_7x5.h>

MatrixDisplay_7x5::MatrixDisplay_7x5( const QString & name ) :
  BreadBoardPart( name ),
  m_segWidth( 16 )
{
  m_rect = QRect( 0, 0, 8+5*m_segWidth, 8+7*m_segWidth );
  
  for (int i=0; i<8; ++i) 
  {
    m_xRegister[i] = m_yRegister[i] = 0;
    m_xValue[i] = m_yValue[i] = 0;
  }
  
  startTimer( 10 );
}

MatrixDisplay_7x5::~MatrixDisplay_7x5()
{
}

void
MatrixDisplay_7x5::paint( QPainter *p )
{
  p->save();
  
  p->setBrush( Qt::gray );
  p->drawRect( rect() );
  p->translate( rect().x(), rect().y() );
  
  for (int y=0; y<7; ++y)
  {
    for (int x=0; x<5; ++x)
    {
      p->setBrush( segmentColor( x, y ) );
      p->drawEllipse( 4+x*m_segWidth+1, 4+y*m_segWidth+1, 
                      m_segWidth-2, m_segWidth-2 );
    }
    
    m_yValue[y] = 0;
  }
  
  for (int x=0; x<5; ++x)
  {
    m_xValue[x] = 0;
  }
  
  p->restore();
}

QColor
MatrixDisplay_7x5::segmentColor( int x, int y )
{
  if (!m_xRegister[x]) return Qt::white;
  if (!m_yRegister[y]) return Qt::white;
  
  if ( (m_xValue[x] & (0x01 << x)) && (m_yValue[y] & (0x01 << y)) )
  {
    return Qt::red;
  }
  
  return Qt::darkRed;
}

void
MatrixDisplay_7x5::assignXRegister( int x, PicRegister *reg, int bit )
{
  m_xRegister[x] = reg;
  m_xBit[x] = bit;
}

void
MatrixDisplay_7x5::assignYRegister( int y, PicRegister *reg, int bit )
{
  m_yRegister[y] = reg;
  m_yBit[y] = bit;
}
  
void
MatrixDisplay_7x5::timerEvent( QTimerEvent * )
{
  for (int y=0; y<5; ++y)
  {
    if (m_yRegister[y])
      m_yValue[y] |= m_yRegister[y]->value();
  }

  for (int x=0; x<7; ++x)
  {
    if (m_xRegister[x])
      m_xValue[x] |= m_xRegister[x]->value();
  }
}

