//======================================================================
// File:		picram.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <nullregister.h>
#include <indfregister.h>
#include <statusregister.h>
#include <pclregister.h>
#include <picram.h>
#include <picregister.h>
#include <pic.h>

PicRAM::PicRAM( int numBanks, Pic *pic, int *pc ) :
  m_nullRegister( new NullRegister )
{
  create( numBanks );
      
  m_bank[0][0] = new INDFRegister( this );
  m_bank[1][0] = m_bank[0][0];
  m_bank[2][0] = m_bank[0][0];
  m_bank[3][0] = m_bank[0][0];
  m_shadow[1][0] = true;
  m_shadow[2][0] = true;
  m_shadow[3][0] = true;
  
  m_bank[0][2] = new PCLRegister( pic, pc );
  m_bank[1][2] = m_bank[0][2];
  m_bank[2][2] = m_bank[0][2];
  m_bank[3][2] = m_bank[0][2];
  m_shadow[1][2] = true;
  m_shadow[2][2] = true;
  m_shadow[3][2] = true;
  
  m_bank[0][3] = new STATUSRegister();
  m_bank[1][3] = m_bank[0][3];
  m_bank[2][3] = m_bank[0][3];
  m_bank[3][3] = m_bank[0][3];
  m_shadow[1][3] = true;
  m_shadow[2][3] = true;
  m_shadow[3][3] = true;
  
  m_bank[0][4] = new PicRegister( PicRegister::FSR, "FSR", 0xff, 0x00 );
  m_bank[1][4] = m_bank[0][4];
  m_bank[2][4] = m_bank[0][4];
  m_bank[3][4] = m_bank[0][4];
  m_shadow[1][4] = true;
  m_shadow[2][4] = true;
  m_shadow[3][4] = true;
  
  m_bank[1][1] = new OPTIONRegister();
  m_bank[3][1] = m_bank[1][1];  
  m_shadow[3][1] = true;
}

PicRAM::~PicRAM()
{
  destroy();
  
  delete m_nullRegister;  
}

PicRegister *
PicRAM::createRegister( int bank, int id, PicRegister::Type type, 
                        const QString & name, int mask, int resetValue )
{
  m_bank[bank][id] = new PicRegister( type, name, mask, resetValue );
  
  return m_bank[bank][id];
}

void
PicRAM::createGeneralPurpose( int bank, int start, int num )
{
  for (int i=0; i<num; ++i)
  {
    m_bank[bank][i+start] = new PicRegister( (bank << 7)+i+start );
  }
}

void 
PicRAM::createShadow( int bank, int start, int num, int sourceBank )
{
  for (int i=0; i<num; ++i)
  {
    const int id = start+i;
    
    m_bank[bank][id] = m_bank[sourceBank][id];
    m_shadow[bank][id] = true;
  }
}

void
PicRAM::create( int numBanks )
{
  m_numBanks = numBanks;
  
  m_bank = new PicRegister ** [numBanks];
  m_shadow = new bool * [numBanks];
  
  for (int bank=0; bank<numBanks; ++bank)
  {
    m_bank[bank] = new PicRegister * [128];
    m_shadow[bank] = new bool [128];
    
    for (int id=0; id<128; ++id)
    {
      m_bank[bank][id] = 0;
      m_shadow[bank][id] = false;
    }
  }
}

void
PicRAM::clearNames()
{
  for (int bank=0; bank<m_numBanks; ++bank)
  {
    for (int id=0; id<128; ++id)
    {
      int addr = 128*bank+id;
      PicRegister *reg = m_bank[bank][id];
      
      if (reg && reg->isGeneralPurpose()) reg->createName( addr );
    }
  }
}

void
PicRAM::reset()
{
  for (int bank=0; bank<m_numBanks; ++bank)
  {
    for (int id=0; id<128; ++id)
    {
      if (m_bank[bank][id]) m_bank[bank][id]->reset();
    }
  }
}

void
PicRAM::freeze()
{
  for (int bank=0; bank<m_numBanks; ++bank)
  {
    for (int id=0; id<128; ++id)
    {
      if (m_bank[bank][id]) m_bank[bank][id]->freeze();
    }
  }
}

int
PicRAM::symbolToAddr( const QString & symbol )
{
  if (symbol.isEmpty()) return -1;
  
  for (int bank=0; bank<m_numBanks; ++bank)
  {
    for (int id=0; id<128; ++id)
    {
      if (m_bank[bank][id] && m_bank[bank][id]->name() == symbol)
      {
        return (bank*128+id);
      }
    }
  }
  
  return -1;
}

void
PicRAM::destroy()
{
  return;
  
  for (int bank=0; bank<m_numBanks; ++bank)
  {
    for (int id=0; id<128; ++id)
    {
      delete m_bank[bank][id];
      
      // avoid double delete for shadows
      //
      m_bank[bank][id] = 0;
    }
    
    delete [] m_bank[bank];
    delete [] m_shadow[bank];
  }
  
  delete [] m_bank;
  delete [] m_shadow;
}
