//======================================================================
// File:		picram.h
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#ifndef PICRAM_HH
#define PICRAM_HH

#include <qstring.h>

#include <picregister.h>
#include <statusregister.h>
#include <optionregister.h>
#include <nullregister.h>

class Pic;

class PicRAM
{
public:
  PicRAM( int numBanks, Pic *pic, int *pc );
  ~PicRAM();
  
  PicRegister *createRegister( int bank, int id, 
                               PicRegister::Type type=PicRegister::GeneralPurpose, 
                               const QString & name=QString::null, 
                               int mask=0xff, int resetValue=0x00 );
  void createGeneralPurpose( int bank, int start, int num );
  void createShadow( int bank, int start, int num, int sourceBank );
  
  int numBanks() const { return m_numBanks; }
  
  PicRegister *get( int addr ) const { return get((addr >> 7),(addr & 0x7f)); }
  PicRegister *get( int bank, int id ) const;
  
  PicRegister *getRegister( int reg ) { return get( status()->bank(), reg ); }
  PicRegister *indf() const { return m_bank[0][0]; }
  PicRegister *fsr() const { return m_bank[0][4]; }
  PicRegister *pcl() const { return m_bank[0][2]; }
  STATUSRegister *status() const { return (STATUSRegister *)m_bank[0][3]; }
  OPTIONRegister *option() const { return (OPTIONRegister *)m_bank[1][1]; }
  PicRegister *null() const { return m_nullRegister; }
  
  PicRegister *rawGet( int bank, int id ) const { return m_bank[bank][id]; }
  
  bool isShadow( int addr ) { return m_shadow[(addr >> 7)][(addr & 0x7f)]; }
  bool isShadow( int bank, int id ) { return m_shadow[bank][id]; }
  
  void reset();
  void freeze();
  
  int symbolToAddr( const QString & symbol );
  
  void setWatchActive() { m_watchActive = true; }
  void resetWatch() { m_watchActive = false; }
  bool watchActive() const { return m_watchActive; }
  
  void clearNames();
  
protected:
  int            m_numBanks;
  PicRegister ***m_bank;
  bool         **m_shadow;
  NullRegister  *m_nullRegister;
  bool           m_watchActive;
  
  void create( int numBanks );
  void destroy();
  
};

inline PicRegister *
PicRAM::get( int bank, int id ) const
{
  PicRegister *ret = m_bank[bank][id];
  
  if (ret) return ret;
  
  return m_nullRegister;
}

#endif // PICRAM_HH
