//======================================================================
// File:		picregister.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <picregister.h>
#include <picram.h>

int PicRegister::m_bitMask[8] = { 1, 2, 4, 8, 16, 32, 64, 128 };
bool PicRegister::watchActive = false;

PicRegister::PicRegister( unsigned addr ) :
  m_value( 0 ),
  m_freezedValue( 0 ),
  m_type( GeneralPurpose ),
  m_mask( 0xff ),
  m_resetValue( 0 ),
  m_watchCondition( 0 ),
  m_readCnt( 0 ),
  m_writeCnt( 0 ),
  m_freezedReadCnt( 0 ),
  m_freezedWriteCnt( 0 )
{
  createName( addr );
  
  createDefaultFlagNames();
}

PicRegister::PicRegister( Type type, const QString & name, 
                          int mask, int resetValue ) :
  m_name( name ),
  m_value( 0 ),
  m_freezedValue( 0 ),
  m_type( type ),
  m_mask( mask ),
  m_resetValue( resetValue ),
  m_watchCondition( 0 ),
  m_readCnt( 0 ),
  m_writeCnt( 0 ),
  m_freezedReadCnt( 0 ),
  m_freezedWriteCnt( 0 )
{
  createDefaultFlagNames();
}

PicRegister::~PicRegister()
{
}

void
PicRegister::createName( int addr )
{
  m_name.sprintf( "0x%03X", addr );
}

void
PicRegister::createDefaultFlagNames()
{
  for (int i=0; i<8; ++i)
  {
    m_flagName[i] = "";
  }
}

void
PicRegister::setFlagNames( const QString & f0, const QString & f1,
                           const QString & f2, const QString & f3,
                           const QString & f4, const QString & f5,
                           const QString & f6, const QString & f7 )
{
  m_flagName[0] = f0;
  m_flagName[1] = f1;
  m_flagName[2] = f2;
  m_flagName[3] = f3;
  m_flagName[4] = f4;
  m_flagName[5] = f5;
  m_flagName[6] = f6;
  m_flagName[7] = f7;
}
                     
  
