//======================================================================
// File:		picregistertableitem.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <qpainter.h>
#include <qlineedit.h>

#include <picregister.h>
#include <picregistertableitem.h>

#include <iostream>

PicRegisterTableItem::PicRegisterTableItem( QTable *table,
                                            PicRegister *reg, 
                                            bool shadow ) :
  HexTableItem( table, reg, 0==reg ? QTableItem::Never : QTableItem::OnTyping ),
  m_colorized( false ),
  m_nullReg( false )
{  
  if (m_reg->type() == PicRegister::Null)
  {
    m_normalColor = Qt::darkGray;
    m_nullReg = true;
  }
  else if (m_reg->isSFR())
  {
    if (shadow)
    {
      m_normalColor = table->colorGroup().background();
    }
    else
    {
      m_normalColor = Qt::green.light(190);
    }
  }
  else
  {
    if (shadow)
    {
      m_normalColor = table->colorGroup().background();
    }
    else
    {
      m_normalColor = Qt::blue.light(190);
    }
  }
  
  m_color = m_normalColor;
  
  update();
}

PicRegisterTableItem::~PicRegisterTableItem()
{
}

void
PicRegisterTableItem::setSpecialColor( const QColor & color ) 
{
  m_color = color;
  m_colorized = true;
}

void
PicRegisterTableItem::resetSpecialColor() 
{
  m_color = m_normalColor;
  m_colorized = false;
}

void
PicRegisterTableItem::paint( QPainter *p, const QColorGroup & /*cg*/, 
                             const QRect & rect, bool )
{
  const QRect cr( 0, 0, rect.width(), rect.height() );
    
  p->fillRect( cr, m_color );
    
  if (m_reg->type() == PicRegister::Null) return;
  
  p->drawText( cr, AlignCenter, text() );
}
