//======================================================================
// File:		picrom.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <picrom.h>
#include <pic.h>

#include <string.h>

PicROM::PicROM( Pic *pic, int size ) :
  m_pic( pic ),
  m_size( 0 ),
  m_data( 0 ),
  m_readCnt( 0 ),
  m_dataCnt( 0 ),
  m_breakpoint( 0 )
{
  setSize( size );
}

PicROM::~PicROM()
{
  freeMemory();
}

void 
PicROM::freeMemory()
{
  if (m_breakpoint)
  {
    for (int i=0; i<m_size; ++i) delete m_breakpoint[i];
  }
  
  delete [] m_data;
  delete [] m_breakpoint;
  delete [] m_readCnt;
}

void
PicROM::setSize( int size )
{
  freeMemory();
  
  m_data = new unsigned [size];
  m_readCnt = new unsigned [size];
  m_breakpoint = new Breakpoint * [size];
  
  m_maxRead = 0;
  
  for (int i=0; i<size; ++i) 
  {
    m_breakpoint[i] = 0;
    m_readCnt[i] = 0;
    m_data[i] = 0;
  }
  
  m_size = size;
}

void
PicROM::setBreakpoint( int addr )
{
  if (m_breakpoint[addr]) return;
  
  m_breakpoint[addr] = new Breakpoint( m_pic->ram() );
}

void
PicROM::removeBreakpoint( int addr )
{
  if (!m_breakpoint[addr]) return;
  
  delete m_breakpoint[addr];
  
  m_breakpoint[addr] = 0;
}

void
PicROM::setData( unsigned *data, int cnt )
{
  memset( m_data, 0, sizeof( unsigned ) * m_size );
  
  m_dataCnt = cnt;
  
  for (int i=0; i<cnt; ++i)
  {
    m_data[i] = data[i];
  }
  
  reset();
}

  
