//======================================================================
// File:		picrom.h
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#ifndef PICROM_HH
#define PICROM_HH

#include <qglobal.h>

#include <breakpoint.h>

class Pic;

class PicROM
{
public:
  PicROM( Pic *, int size=2048 );
  ~PicROM();
  
  void setSize( int size );
  int size() const { return m_size; }
  int dataCnt() const { return m_dataCnt; }
  
  unsigned read( int );

  void reset();
  
  int interruptVector() const { return 4; }
  int resetVector() const { return 0; }
  
  bool isActiveBreakpoint( int i ) const;
  bool isBreakpoint( int i ) const { return (m_breakpoint[i] != 0); }
  bool temporaryBreakpoint( int i ) const { return m_breakpoint[i]->isTemporary(); }
  bool breakpointEnabled( int i ) const { return m_breakpoint[i]->isEnabled(); }
  bool breakpointConditional( int i ) const { return m_breakpoint[i]->isConditional(); }
  void setBreakpoint( int );
  void removeBreakpoint( int );
  Breakpoint *breakpoint( int addr ) const { return m_breakpoint[addr]; }
  
  void setData( unsigned *, int );

  unsigned at( int i ) const { return m_data[i]; }
  unsigned readCntAt( int i ) const { return m_readCnt[i]; }
  unsigned maxRead() const { return m_maxRead; }
  
  void startCall( int );
  void endCall( int );
  void minCall() const;
  void maxCall() const;
  
protected:
  Pic                *m_pic;
  int                 m_size;
  unsigned           *m_data;
  unsigned           *m_readCnt;
  unsigned            m_maxRead;
  int                 m_dataCnt;
  Breakpoint        **m_breakpoint;
  
  void freeMemory();
  
};

inline bool 
PicROM::isActiveBreakpoint( int addr ) const 
{ 
  const Breakpoint *bp = breakpoint( addr );
  
  return (bp && bp->isEnabled() && bp->match());
}

inline void
PicROM::reset()
{
  m_maxRead = 0;
  
  for (int i=0; i<m_dataCnt; ++i)
  {
    m_readCnt[i] = 0;
  }
}

inline unsigned
PicROM::read( int pc )
{
  m_readCnt[pc]++;
  if (m_readCnt[pc] > m_maxRead) m_maxRead = m_readCnt[pc];
  
  return m_data[pc];
}
    
#endif // PICROM_HH
