//======================================================================
// File:		picstack.h
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#ifndef PICSTACK_HH
#define PICSTACK_HH

class PicStack
{
public:
  PicStack( int size=8 );
  ~PicStack();
  
  void push( unsigned );
  unsigned pop();
  unsigned top() const;
  
  unsigned at( int i ) const { return m_stack[i]; }
  int stackPointer() const { return m_stackPointer; }
  int size() const { return m_size; }
  
  void reset();
  
  unsigned long cyclesAtCall() const { return m_cyclesAtCall[m_stackPointer]; }
  void setCyclesAtCall( unsigned long cycles );
  
  unsigned pcAtCall() const { return m_pcAtCall[m_stackPointer]; }
  void setPcAtCall( unsigned pc );
  
  unsigned call() const { return m_call[m_stackPointer]; }
  void setCall( unsigned pc ) { m_call[m_stackPointer] = pc; }
  unsigned callAt( int i ) const { return m_call[i]; }
  
protected:
  const int      m_size;
  unsigned      *m_stack;
  unsigned      *m_call;
  unsigned long *m_cyclesAtCall;
  int           *m_pcAtCall;
  int            m_stackPointer;
  int            m_mask;
  
};

inline void
PicStack::setCyclesAtCall( unsigned long cycles )
{ 
  m_cyclesAtCall[m_stackPointer] = cycles; 
}

inline void
PicStack::setPcAtCall( unsigned pc )
{ 
  m_pcAtCall[m_stackPointer] = pc; 
}

inline void 
PicStack::push( unsigned addr )
{
  m_stack[m_stackPointer++] = addr;
  
  m_stackPointer &= m_mask;
}

inline unsigned 
PicStack::pop()
{
  --m_stackPointer;
  m_stackPointer += m_size;
  m_stackPointer &= m_mask;
  
  return m_stack[m_stackPointer];
}

inline unsigned
PicStack::top() const
{
  return m_stack[(m_stackPointer-1+m_size)&m_mask];
}

#endif // PICSTACK_HH
