//======================================================================
// File:		picsyntaxhighlighter.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <picsyntaxhighlighter.h>
#include <qtextedit.h>

#include <iostream>

QStringList PicSyntaxHighlighter::s_keywords;
QStringList PicSyntaxHighlighter::s_prepro;
QStringList PicSyntaxHighlighter::s_sfr;
QStringList PicSyntaxHighlighter::s_bits;
bool PicSyntaxHighlighter::s_initialized;
QFont PicSyntaxHighlighter::s_font[NumCategory];
QColor PicSyntaxHighlighter::s_color[NumCategory];

PicSyntaxHighlighter::PicSyntaxHighlighter( QTextEdit *edit ) : 
  QSyntaxHighlighter( edit )
{
  initializeStaticStuff( edit );
}

PicSyntaxHighlighter::~PicSyntaxHighlighter()
{
}

int
PicSyntaxHighlighter::highlightParagraph( const QString & text, int )
{
  QStringList tokenList;
  
  unsigned pos=0;
  QString token;
  
  bool whiteSpace = isSpace( text[0] );
  
  while (pos < text.length())
  {
    const QChar currrent = text[pos];
    
    // check if comment
    //
    if (currrent == ';')
    {
      // append last token
      //
      appendToListAndTruncate( token, tokenList );
      
      // comment is one token
      //
      tokenList.append( text.mid( pos ) );
      break;
    }
    
    if (whiteSpace)
    {
      if (!isSpace(currrent))
      {
        appendToListAndTruncate( token, tokenList );
        whiteSpace = false;
      }
    }
    else
    {
      if (isSpace(currrent))
      {
        appendToListAndTruncate( token, tokenList );        
        whiteSpace = true;
      }
    }
    
    token += currrent;
    
    pos++;
  }
  
  appendToListAndTruncate( token, tokenList );
/*  
  std::cerr << "text: " << text.latin1() << std::endl;
  std::cerr << "tokens: " << tokenList.count() << std::endl;
  for (int i=0; i<tokenList.count(); ++i)
  {
    std::cerr << "(" << tokenList[i].latin1() << ")" << std::endl;
  }
*/  
  pos = 0;
  colorize( 0, text.length(), Normal );
  
  for (QStringList::Iterator it=tokenList.begin(); it != tokenList.end(); ++it)
  {
    token = *it;
    
    if (!isSpace(token[0]))
    {
      if (token[0] == ';')
      {
        colorize( pos, token.length(), Comment );
      }
      else
      {
        // keyword (assembler statement) ?
        //
        if (isKeyword(token))
        {
          colorize( pos, token.length(), Keyword );
        }
        
        // known sfr ?
        //
        else if (isSFR(token))
        {
          colorize( pos, token.length(), SFR );
        }
        
        // known bit ?
        //
        else if (isBit(token))
        {
          colorize( pos, token.length(), Bits );
        }
        
        // preprocessor statement ?
        //
        else if (isPrePro(token))
        {
          colorize( pos, token.length(), PrePro );
        }
        
        // number ?
        //
        else if (isNumber(token))
        {
          colorize( pos, token.length(), Number );
        }
      }
    }
    
    pos += token.length();
  }
  
  return 0;
}

bool
PicSyntaxHighlighter::isKeyword( const QString & token ) const
{
  for (unsigned i=0; i<s_keywords.count(); ++i)
  {
    if (token == s_keywords[i]) return true;
  }
  
  return false;
}

bool
PicSyntaxHighlighter::isPrePro( const QString & token ) const
{
  for (unsigned i=0; i<s_prepro.count(); ++i)
  {
    if (token == s_prepro[i]) return true;
  }
  
  return false;
}

bool
PicSyntaxHighlighter::isSFR( const QString & token ) const
{
  for (unsigned i=0; i<s_sfr.count(); ++i)
  {
    if (token == s_sfr[i]) return true;
  }
  
  return false;
}

// Bitte ein Bit
//
bool
PicSyntaxHighlighter::isBit( const QString & token ) const
{
  for (unsigned i=0; i<s_bits.count(); ++i)
  {
    if (token == s_bits[i]) return true;
  }
  
  return false;
}

bool
PicSyntaxHighlighter::isNumber( const QString & token ) const
{
  if (token.startsWith( "0x" ))
  {
    for (unsigned i=2; i<token.length(); ++i)
    {
      if (!isHex(token[i])) return false;
    }
  }
  else if (token.startsWith( "b'" ) && token[token.length()-1] == '\'')
  {
    for (unsigned i=2; i<token.length()-1; ++i)
    {
      if (token[i] != '1' && token[i] != '0') return false;
    }
  }
  else if (token.startsWith( "d'" ) && token[token.length()-1] == '\'')
  {
    for (unsigned i=2; i<token.length()-1; ++i)
    {
      if (!token[i].isNumber()) return false;
    }
  }
  else if (token.startsWith( "h'" ) && token[token.length()-1] == '\'')
  {
    for (unsigned i=2; i<token.length()-1; ++i)
    {
      if (!isHex(token[i])) return false;
    }
  }
  else
  {
    for (unsigned i=0; i<token.length(); ++i)
    {
      if (!token[i].isNumber()) return false;
    }
  }
    
  return true;
}

void
PicSyntaxHighlighter::initializeStaticStuff( QTextEdit *edit )
{
  if (!s_initialized)
  {
    // keywords
    //
    s_keywords.append( "$" );
    s_keywords.append( "addlw" );
    s_keywords.append( "addwf" );
    s_keywords.append( "andlw" );
    s_keywords.append( "andwf" );
    s_keywords.append( "b" );
    s_keywords.append( "bc" );
    s_keywords.append( "bcf" );
    s_keywords.append( "bnz" );
    s_keywords.append( "bsf" );
    s_keywords.append( "btfsc" );
    s_keywords.append( "btfss" );
    s_keywords.append( "bz" );
    s_keywords.append( "call" );
    s_keywords.append( "clrc" );
    s_keywords.append( "clrdc" );
    s_keywords.append( "clrf" );
    s_keywords.append( "clrw" );
    s_keywords.append( "clrwdt" );
    s_keywords.append( "clrz" );
    s_keywords.append( "comf" );
    s_keywords.append( "decf" );
    s_keywords.append( "decfsz" );
    s_keywords.append( "goto" );
    s_keywords.append( "incf" );
    s_keywords.append( "incfsz" );
    s_keywords.append( "iorlw" );
    s_keywords.append( "iorwf" );
    s_keywords.append( "movf" );
    s_keywords.append( "movfw" );
    s_keywords.append( "movlw" );
    s_keywords.append( "movwf" );
    s_keywords.append( "nop" );
    s_keywords.append( "option" );
    s_keywords.append( "retfie" );
    s_keywords.append( "retlw" );
    s_keywords.append( "return" );
    s_keywords.append( "rlf" );
    s_keywords.append( "rrf" );
    s_keywords.append( "setc" );
    s_keywords.append( "setdc" );
    s_keywords.append( "setz" );
    s_keywords.append( "skpc" );
    s_keywords.append( "skpdc" );
    s_keywords.append( "skpnc" );
    s_keywords.append( "skpndc" );
    s_keywords.append( "skpnz" );
    s_keywords.append( "skpz" );
    s_keywords.append( "sleep" );
    s_keywords.append( "sublw" );
    s_keywords.append( "subwf" );
    s_keywords.append( "swapf" );
    s_keywords.append( "tris" );
    s_keywords.append( "tstf" );
    s_keywords.append( "xorlw" );
    s_keywords.append( "xorwf" );
  
    // Preprocessor
    //
    s_prepro.append( "#define" );
    s_prepro.append( "__badram" );
    s_prepro.append( "__config" );
    s_prepro.append( "__idlocs" );
    s_prepro.append( "__maxram" );
    s_prepro.append( "_bod_nsleep" );  
    s_prepro.append( "_bod_off" );     
    s_prepro.append( "_bod_on" );      
    s_prepro.append( "_bod_sboden" );  
    s_prepro.append( "_boden" );              
    s_prepro.append( "_boden_off" );            
    s_prepro.append( "_boden_on" );             
    s_prepro.append( "_boren_off" );            
    s_prepro.append( "_boren_on" );             
    s_prepro.append( "_cp" );                 
    s_prepro.append( "_cp_50" ); 
    s_prepro.append( "_cp_75" ); 
    s_prepro.append( "_cp_all" );
    s_prepro.append( "_cp_off" );               
    s_prepro.append( "_cp_on" );                
    s_prepro.append( "_cpd" );                
    s_prepro.append( "_cpd_off" );            
    s_prepro.append( "_cpd_on" );      
    s_prepro.append( "_data_cp_off" );          
    s_prepro.append( "_data_cp_on" );           
    s_prepro.append( "_dp_off" );
    s_prepro.append( "_dp_on" ); 
    s_prepro.append( "_ec_osc" );             
    s_prepro.append( "_er_osc_clkout" );        
    s_prepro.append( "_er_osc_noclkout" );      
    s_prepro.append( "_extclk_osc" );           
    s_prepro.append( "_extrc_osc_clkout" );   
    s_prepro.append( "_extrc_osc_noclkout" ); 
    s_prepro.append( "_fcmen_off" );   
    s_prepro.append( "_fcmen_on" );    
    s_prepro.append( "_hs_osc" );               
    s_prepro.append( "_ieso_off" );    
    s_prepro.append( "_ieso_on" );     
    s_prepro.append( "_intosc_osc_clkout" );   	
    s_prepro.append( "_intosc_osc_noclkout" );  
    s_prepro.append( "_intrc_osc_clkout" );   	
    s_prepro.append( "_intrc_osc_noclkout" );	  
    s_prepro.append( "_lp_osc" );              
    s_prepro.append( "_lvp_off" );              
    s_prepro.append( "_lvp_on" );               
    s_prepro.append( "_mclre_off" );            
    s_prepro.append( "_mclre_on" );             
    s_prepro.append( "_pwrte_off" );            
    s_prepro.append( "_pwrte_on" );             
    s_prepro.append( "_rc_osc" );
    s_prepro.append( "_rc_osc_clkout" );        
    s_prepro.append( "_rc_osc_noclkout" );      
    s_prepro.append( "_wdt_off" );              
    s_prepro.append( "_wdt_on" );               
    s_prepro.append( "_xt_osc" );               
    s_prepro.append( "bankisel" );
    s_prepro.append( "banksel" );
    s_prepro.append( "cblock" );
    s_prepro.append( "code" );
    s_prepro.append( "constant" );
    s_prepro.append( "da" );
    s_prepro.append( "data" );
    s_prepro.append( "db" );
    s_prepro.append( "de" );
    s_prepro.append( "dt" );
    s_prepro.append( "dw" );
    s_prepro.append( "else" );
    s_prepro.append( "end" );
    s_prepro.append( "endc" );
    s_prepro.append( "endif" );
    s_prepro.append( "endm" );
    s_prepro.append( "endw" );
    s_prepro.append( "equ" );
    s_prepro.append( "error" );
    s_prepro.append( "errorlevel" );
    s_prepro.append( "exitm" );
    s_prepro.append( "expand" );
    s_prepro.append( "extern" );
    s_prepro.append( "fill" );
    s_prepro.append( "global" );
    s_prepro.append( "idata" );
    s_prepro.append( "if" );
    s_prepro.append( "ifdef" );
    s_prepro.append( "ifndef" );
    s_prepro.append( "include" );
    s_prepro.append( "list" );
    s_prepro.append( "local" );
    s_prepro.append( "macro" );
    s_prepro.append( "messg" );
    s_prepro.append( "noexpand" );
    s_prepro.append( "nolist" );
    s_prepro.append( "org" );
    s_prepro.append( "page" );
    s_prepro.append( "pagesel" );
    s_prepro.append( "processor" );
    s_prepro.append( "radix" );
    s_prepro.append( "res" );
    s_prepro.append( "set" );
    s_prepro.append( "space" );
    s_prepro.append( "subtitle" );
    s_prepro.append( "title" );
    s_prepro.append( "udata" );
    s_prepro.append( "udata_acs" );
    s_prepro.append( "udata_ovr" );
    s_prepro.append( "udata_shr" );
    s_prepro.append( "variable" );
    s_prepro.append( "while" );
  
    // defines
    //
    s_sfr.append( "adcon0" );
    s_sfr.append( "adcon1" );
    s_sfr.append( "adcond0" );
    s_sfr.append( "adres" );
    s_sfr.append( "adresh" );
    s_sfr.append( "adresl" );
    s_sfr.append( "ansel" );	
    s_sfr.append( "ccp1con" );
    s_sfr.append( "ccp2con" );
    s_sfr.append( "ccpr1h" );
    s_sfr.append( "ccpr1l" );
    s_sfr.append( "ccpr2h" );
    s_sfr.append( "ccpr2l" );
    s_sfr.append( "cmcon" );
    s_sfr.append( "cmcon0" );	
    s_sfr.append( "cmcon1" );	
    s_sfr.append( "eccpas" );	 
    s_sfr.append( "eeadr" );
    s_sfr.append( "eecon1" );
    s_sfr.append( "eecon2" );
    s_sfr.append( "eedat" );		
    s_sfr.append( "eedata" );
    s_sfr.append( "fsr" );
    s_sfr.append( "indf" );
    s_sfr.append( "intcon" );
    s_sfr.append( "ioc" );		
    s_sfr.append( "ioca" );	
    s_sfr.append( "option_reg" );
    s_sfr.append( "oscal" );
    s_sfr.append( "osccal" );
    s_sfr.append( "osccon" );	
    s_sfr.append( "osctune" );	
    s_sfr.append( "pcl" );
    s_sfr.append( "pclath" );
    s_sfr.append( "pcon" );
    s_sfr.append( "pie1" );
    s_sfr.append( "pir1" );
    s_sfr.append( "porta" );
    s_sfr.append( "portb" );
    s_sfr.append( "portc" );  
    s_sfr.append( "pr2" );
    s_sfr.append( "pwm1con" );	
    s_sfr.append( "rcreg" );
    s_sfr.append( "rcsta" );
    s_sfr.append( "spbrg" );
    s_sfr.append( "sspadd" );
    s_sfr.append( "sspatat" );
    s_sfr.append( "sspbuf" );
    s_sfr.append( "sspcon" );
    s_sfr.append( "status" );
    s_sfr.append( "t1con" );
    s_sfr.append( "t2con" );
    s_sfr.append( "tmr0" );
    s_sfr.append( "tmr1h" );
    s_sfr.append( "tmr1l" );
    s_sfr.append( "tmr2" );
    s_sfr.append( "trisa" );
    s_sfr.append( "trisb" );
    s_sfr.append( "trisc" );	
    s_sfr.append( "txreg" );
    s_sfr.append( "txsta" );
    s_sfr.append( "vrcon" );
    s_sfr.append( "wdtcon" );	 
    s_sfr.append( "wpu" );		
    s_sfr.append( "wpua" );	
    
    // bits
    //
    s_bits.append( "adcs0" );
    s_bits.append( "adcs1" );
    s_bits.append( "adcs2" );
    s_bits.append( "aden" );
    s_bits.append( "adfm" );     
    s_bits.append( "adie" );     
    s_bits.append( "adif" );     
    s_bits.append( "adon" );     
    s_bits.append( "ans0" ); 
    s_bits.append( "ans1" ); 
    s_bits.append( "ans2" ); 
    s_bits.append( "ans3" ); 
    s_bits.append( "ans4" ); 
    s_bits.append( "ans5" ); 
    s_bits.append( "ans6" ); 
    s_bits.append( "ans7" ); 
    s_bits.append( "brgh" );       
    s_bits.append( "c" );
    s_bits.append( "c1ie" );     
    s_bits.append( "c1if" );     
    s_bits.append( "c1inv" );      
    s_bits.append( "c1out" );      
    s_bits.append( "c2ie" );     
    s_bits.append( "c2if" );     
    s_bits.append( "c2inv" );      
    s_bits.append( "c2out" );      
    s_bits.append( "c2sync" );   
    s_bits.append( "cal0" );     
    s_bits.append( "cal1" );     
    s_bits.append( "cal2" );     
    s_bits.append( "cal3" );     
    s_bits.append( "cal4" );     
    s_bits.append( "cal5" );     
    s_bits.append( "ccp1ie" );     
    s_bits.append( "ccp1if" );
    s_bits.append( "ccp1m0" );
    s_bits.append( "ccp1m1" );
    s_bits.append( "ccp1m2" );
    s_bits.append( "ccp1m3" );
    s_bits.append( "ccp1x" );
    s_bits.append( "ccp1y" );
    s_bits.append( "chs0" );     
    s_bits.append( "chs1" );     
    s_bits.append( "chs2" );     
    s_bits.append( "cinv" );     
    s_bits.append( "cis" );        
    s_bits.append( "cm0" );        
    s_bits.append( "cm1" );        
    s_bits.append( "cm2" );        
    s_bits.append( "cmie" );       
    s_bits.append( "cmif" );
    s_bits.append( "cout" );     
    s_bits.append( "cren" );
    s_bits.append( "csrc" );       
    s_bits.append( "dc" );
    s_bits.append( "dc1b0" );    
    s_bits.append( "dc1b1" );    
    s_bits.append( "eccpas0" );  
    s_bits.append( "eccpas1" );  
    s_bits.append( "eccpas2" );  
    s_bits.append( "eccpase" );  
    s_bits.append( "eeie" );       
    s_bits.append( "eeif" );
    s_bits.append( "f" );
    s_bits.append( "ferr" );
    s_bits.append( "gie" );
    s_bits.append( "go" );       
    s_bits.append( "go_done" );  
    s_bits.append( "hts" );      
    s_bits.append( "inte" );
    s_bits.append( "intedg" );     
    s_bits.append( "intf" );
    s_bits.append( "ioc0" );     
    s_bits.append( "ioc1" );     
    s_bits.append( "ioc2" );     
    s_bits.append( "ioc3" );     
    s_bits.append( "ioc4" );     
    s_bits.append( "ioc5" );     
    s_bits.append( "ioca0" );    
    s_bits.append( "ioca1" );    
    s_bits.append( "ioca2" );    
    s_bits.append( "ioca3" );    
    s_bits.append( "ioca4" );    
    s_bits.append( "ioca5" );    
    s_bits.append( "ircf0" );    
    s_bits.append( "ircf1" );    
    s_bits.append( "ircf2" );    
    s_bits.append( "irp" );
    s_bits.append( "lts" );      
    s_bits.append( "not_bo" );     
    s_bits.append( "not_bod" );    
    s_bits.append( "not_bor" );    
    s_bits.append( "not_done" ); 
    s_bits.append( "not_gppu" );
    s_bits.append( "not_pd" );
    s_bits.append( "not_por" );    
    s_bits.append( "not_rapu" ); 
    s_bits.append( "not_rbpu" );   
    s_bits.append( "not_t1sync" );
    s_bits.append( "not_to" );
    s_bits.append( "oerr" );
    s_bits.append( "oscf" );       
    s_bits.append( "osfie" );    
    s_bits.append( "osfif" );    
    s_bits.append( "osts" );     
    s_bits.append( "p1m0" );     
    s_bits.append( "p1m1" );     
    s_bits.append( "pa0" );
    s_bits.append( "pa1" );
    s_bits.append( "pa2" );
    s_bits.append( "pdc0" );     
    s_bits.append( "pdc1" );     
    s_bits.append( "pdc2" );     
    s_bits.append( "pdc3" );     
    s_bits.append( "pdc4" );     
    s_bits.append( "pdc5" );     
    s_bits.append( "pdc6" );     
    s_bits.append( "peie" );
    s_bits.append( "prsen" );    
    s_bits.append( "ps0" );        
    s_bits.append( "ps1" );        
    s_bits.append( "ps2" );        
    s_bits.append( "psa" );        
    s_bits.append( "pssac0" );   
    s_bits.append( "pssac1" );   
    s_bits.append( "pssbd0" );   
    s_bits.append( "pssbd1" );   
    s_bits.append( "raie" );     
    s_bits.append( "raif" );     
    s_bits.append( "rbie" );
    s_bits.append( "rbif" );
    s_bits.append( "rcie" );       
    s_bits.append( "rcif" );
    s_bits.append( "rd" );         
    s_bits.append( "rp0" );
    s_bits.append( "rp1" );
    s_bits.append( "rx9" );
    s_bits.append( "rx9d" );
    s_bits.append( "sboden" );   
    s_bits.append( "scs" );      
    s_bits.append( "spen" );
    s_bits.append( "sren" );
    s_bits.append( "swdten" );   
    s_bits.append( "sync" );       
    s_bits.append( "t0cs" );       
    s_bits.append( "t0ie" );
    s_bits.append( "t0if" );     
    s_bits.append( "t0if" );        
    s_bits.append( "t0se" );       
    s_bits.append( "t1ckps0" );
    s_bits.append( "t1ckps1" );
    s_bits.append( "t1ginv" );   
    s_bits.append( "t1gss" );    
    s_bits.append( "t1ie" );     
    s_bits.append( "t1if" );     
    s_bits.append( "t1oscen" );
    s_bits.append( "t2ckps0" ); 
    s_bits.append( "t2ckps1" ); 
    s_bits.append( "t2ie" );     
    s_bits.append( "t2if" );     
    s_bits.append( "tmr1cs" );
    s_bits.append( "tmr1ge" );   
    s_bits.append( "tmr1ie" );     
    s_bits.append( "tmr1if" );
    s_bits.append( "tmr1on" );  
    s_bits.append( "tmr2ie" );     
    s_bits.append( "tmr2if" );
    s_bits.append( "tmr2on" );  
    s_bits.append( "to1f" );
    s_bits.append( "toutps0" ); 
    s_bits.append( "toutps1" ); 
    s_bits.append( "toutps2" ); 
    s_bits.append( "toutps3" ); 
    s_bits.append( "trmt" );       
    s_bits.append( "tun0" );     
    s_bits.append( "tun1" );     
    s_bits.append( "tun2" );     
    s_bits.append( "tun3" );     
    s_bits.append( "tun4" );     
    s_bits.append( "tx9" );        
    s_bits.append( "tx9d" );       
    s_bits.append( "txen" );       
    s_bits.append( "txie" );       
    s_bits.append( "txif" );
    s_bits.append( "ulpwue" );   
    s_bits.append( "vcfg" );     
    s_bits.append( "vr0" );        
    s_bits.append( "vr1" );        
    s_bits.append( "vr2" );        
    s_bits.append( "vr3" );        
    s_bits.append( "vren" );       
    s_bits.append( "vroe" );       
    s_bits.append( "vrr" );        
    s_bits.append( "w" );
    s_bits.append( "wdtps0" );   
    s_bits.append( "wdtps1" );   
    s_bits.append( "wdtps2" );   
    s_bits.append( "wdtps3" );   
    s_bits.append( "wr" );         
    s_bits.append( "wren" );       
    s_bits.append( "wrerr" );      
    s_bits.append( "z" );
    
    s_font[Normal]  = edit->font();
    s_font[Comment] = edit->font();
    //s_font[Comment].setItalic( true );
    s_font[Number]  = edit->font();
    s_font[PrePro]  = edit->font();
    s_font[Keyword] = edit->font();
    s_font[SFR]     = edit->font();
    s_font[Bits]    = edit->font();

    s_color[Normal]  = edit->colorGroup().text();
    s_color[Comment] = Qt::darkGray;
    s_color[Number]  = Qt::darkGreen;
    s_color[PrePro]  = Qt::blue;
    s_color[Keyword] = Qt::darkRed;
    s_color[SFR]     = Qt::darkMagenta;
    s_color[Bits]    = Qt::darkCyan;

    s_initialized = true;
  }
}

