//======================================================================
// File:		picsyntaxhighlighter.h
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#ifndef PICSYNTAXHIGHLIGHTER_HH
#define PICSYNTAXHIGHLIGHTER_HH

#include <qsyntaxhighlighter.h>
#include <qstringlist.h>

class QTextEdit;

class PicSyntaxHighlighter : public QSyntaxHighlighter
{
public:
  enum Category
  {
    Normal = 0,
    Keyword,
    Comment,
    PrePro,
    Number,
    SFR,
    Bits,
    NumCategory
  };
    
  PicSyntaxHighlighter( QTextEdit * );
  virtual ~PicSyntaxHighlighter();

  virtual int highlightParagraph( const QString & text, int );

protected:
  static bool        s_initialized;
  static QStringList s_keywords;
  static QStringList s_prepro;
  static QStringList s_sfr;
  static QStringList s_bits;
  static QFont       s_font[NumCategory];
  static QColor      s_color[NumCategory];
    
  bool isKeyword( const QString & ) const;
  bool isPrePro( const QString & ) const;
  bool isSFR( const QString & ) const;
  bool isBit( const QString & ) const;
  bool isNumber( const QString & ) const;
  bool isSpace( const QChar & ) const;
  bool isHex( const QChar & ) const;
  
  void appendToListAndTruncate( QString & token, QStringList & list );
  void colorize( int pos, int length, Category );
 
  void initializeStaticStuff( QTextEdit * );
  
};

inline void
PicSyntaxHighlighter::appendToListAndTruncate( QString & token,
                                               QStringList & list )
{
  if (!token.isEmpty())
  {
    list.append( token.lower() );
    token = QString::null;
  }
}

inline void
PicSyntaxHighlighter::colorize( int pos, int length, Category category )
{
  setFormat( pos, length, s_font[category], s_color[category] );
}

inline bool
PicSyntaxHighlighter::isHex( const QChar & c ) const
{
  if (c.isNumber() ||
      c == 'a'     || 
      c == 'b'     || 
      c == 'c'     || 
      c == 'd'     || 
      c == 'e'     || 
      c == 'f') return true;
  
  return false;
}

inline bool
PicSyntaxHighlighter::isSpace( const QChar & c ) const
{
  if (c.isSpace() || 
      c == ','    ||
      c == '-'    ||
      c == '+') return true;
  
  return false;
}

#endif // PICSYNTAXHIGHLIGHTER_HH
