//======================================================================
// File:		pictimer0.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <picregister.h>
#include <optionregister.h>
#include <pictimer0.h>
#include <pic.h>

#include <iostream>

PicTimer0::PicTimer0( Pic *pic ) :
  PicTimer( "TIMER0" ),
  m_pic( pic ),
  m_reg( pic->sfr( Pic::TMR0 ) ),
  m_optionReg( pic->ram()->option() )
{
  reset();
}

PicTimer0::~PicTimer0()
{
}

void
PicTimer0::reset()
{
  m_prescaleCnt = m_optionReg->prescaleTimer0();
  m_value = 0;
}

void
PicTimer0::clockTick()
{
  if (m_optionReg->t0cs()) return;
  
  if (--m_prescaleCnt <= 0)
  {
    m_value++;
    m_prescaleCnt = m_optionReg->prescaleTimer0();
  }
  
  if (m_value > 0xff)
  {
    m_value = 0;
    
    if (m_pic->sfr( Pic::INTCON )->bitSetPic( 5 ))    // T0IE
    {
      if (m_pic->sfr( Pic::INTCON )->bitSetPic( 7 ))  // GIE
      {
        m_pic->sfr( Pic::INTCON )->setBitPic( 2 );    // T0IF
        
        m_pic->stack()->setCall( 4 );
        m_pic->stack()->push( m_pic->pc() );
        m_pic->setPc( 4, false );
        m_pic->flushInstructionPipeline();
        m_pic->fetchInstruction( 4 );
      }
    }
  }
  
  setRegisterInternal();
}

