//======================================================================
// File:		ramview.h
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#ifndef RAMVIEW_HH
#define RAMVIEW_HH

#include <uiramview.h>

#include <pic.h>

class QPopupMenu;
class HighlightCondition;

class RAMView : public UIRAMView
{
  Q_OBJECT
public:
  enum ColorItem
  {
    ColorRed=0,
    ColorGreen,
    ColorBlue,
    ColorCyan,
    ColorMagenta,
    ColorYellow,
    NumColors
  };
    
  enum PopupId
  {
    ADD_WATCH_ID,
    REMOVE_WATCH_ID,
    ADD_TRACE_ID,
    REMOVE_TRACE_ID
  };
    
  RAMView( QWidget *parent=0, const char *name=0 );
  virtual ~RAMView();
  
  void setPic( Pic *pic );
  void setEditable( bool );
  
signals:
  void addWatch( PicRegister * );
  void removeWatch( PicRegister * );
  void addTrace( PicRegister * );
  void removeTrace( PicRegister * );
  void updateViews();
  
public slots:
  void refreshSLOT();
  void registerChangedSLOT( PicRegister * );
  void show();
  void registerSelectedSLOT( int );
  
protected:
  Pic                 *m_pic;
  PicRegister         *m_currentReg;
  HighlightCondition  *m_condition;
  QColor               m_colors[6];
  int                 *m_lastValue;
  QPopupMenu          *m_popup;
  QLineEdit           *m_edit[6];
  
  void setValue( int value );
  void setColors();
  void valueChanged( const QString & text, int base );
  void colorChanged( const QString & pattern, ColorItem item );
  
  void resizeEvent( QResizeEvent * );
  
protected slots:
  void currentChangedSLOT( int row, int col );
  void currentChangedSLOT( PicRegister *reg );
  void hexChangedSLOT( const QString & text );
  void decChangedSLOT( const QString & text );
  void binChangedSLOT( const QString & text );
  void redChangedSLOT( const QString & text );
  void greenChangedSLOT( const QString & text );
  void blueChangedSLOT( const QString & text );
  void cyanChangedSLOT( const QString & text );
  void magentaChangedSLOT( const QString & text );
  void yellowChangedSLOT( const QString & text );
  void pressedSLOT( int, int, int button, const QPoint & );
  void popupSLOT( int );
  
};

#endif // RAMVIEW_HH
