//======================================================================
// File:		registertrace.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <pic.h>
#include <registertrace.h>

RegisterTrace::RegisterTrace( Pic *pic ) :
  m_pic( pic )
{
  m_traceBufferList.setAutoDelete( true );
  
  m_it = new RegisterTraceBufferListIterator( m_traceBufferList ); 
}

RegisterTrace::~RegisterTrace()
{
  delete m_it;
}

void
RegisterTrace::addRegister( PicRegister *reg )
{
  m_pic->lock();
  reset();
  m_traceBufferList.append( new RegisterTraceBuffer( reg ) );
  reg->setTraced( true );
  m_pic->unlock();
}

void
RegisterTrace::removeRegister( PicRegister *reg )
{
  m_pic->lock();
  
  for (m_it->toFirst(); **m_it; ++(*m_it))
  {
    if ((**m_it)->reg() == reg)
    {
      m_traceBufferList.remove( **m_it );
      reg->setTraced( false );
      break;
    }
  }
  
  m_pic->unlock();
}

void
RegisterTrace::trace()
{
  if (m_it->isEmpty()) return;
  
  for (m_it->toFirst(); **m_it; ++(*m_it))
  {
    (**m_it)->add( m_pic->cycles() );
  }
}

void
RegisterTrace::reset()
{
  for (m_it->toFirst(); **m_it; ++(*m_it))
  {
    (**m_it)->reset();
  }
}
