//======================================================================
// File:		romview.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <qheader.h>
#include <qlabel.h>
#include <qpixmap.h>

#include <pic.h>
#include <picregistertableitem.h>
#include <romview.h>
#include <romviewtable.h>

#include <iostream>

ROMView::ROMView( QWidget *parent, const char *name ) :
  UIROMView( parent, name ),
  m_lastPC( 0 )
{
  connect( ui_view, SIGNAL( refresh() ),
           this, SLOT( refreshSLOT() ));
  connect( ui_view, SIGNAL( showBreakpoint( int ) ),
           this, SIGNAL( showBreakpoint( int ) ));
  connect( ui_view, SIGNAL( newBreakpoint( int ) ),
           this, SIGNAL( newBreakpoint( int ) ));
  connect( ui_view, SIGNAL( changedBreakpoint( int ) ),
           this, SIGNAL( changedBreakpoint( int ) ));
  connect( ui_view, SIGNAL( removeBreakpoint( int ) ),
           this, SIGNAL( removeBreakpoint( int ) ));
}

ROMView::~ROMView()
{
}

void
ROMView::show()
{
  UIROMView::show();
  refreshSLOT();
}

void
ROMView::setPic( Pic * pic )
{
  m_pic = pic;
  
  if (m_pic)
  {
    QString addr;
    QString opcode;
    QString mnemonic;
    QString cnt;
    
    ui_view->setPic( m_pic );
    
    for (int i=0; i<m_pic->rom()->size(); ++i)
    {
      addr.sprintf( "%03x", i );    
      unsigned opc = m_pic->rom()->at(i);
      
      if (opc != 0)
      {
        opcode.sprintf( "%04X : %c%c %c%c%c%c %c%c%c%c %c%c%c%c ", opc,
                        opc & 0x2000 ? '1' : '0',
                        opc & 0x1000 ? '1' : '0',
                        opc & 0x0800 ? '1' : '0',
                        opc & 0x0400 ? '1' : '0',
                        opc & 0x0200 ? '1' : '0',
                        opc & 0x0100 ? '1' : '0',
                        opc & 0x0080 ? '1' : '0',
                        opc & 0x0040 ? '1' : '0',
                        opc & 0x0020 ? '1' : '0',
                        opc & 0x0010 ? '1' : '0',
                        opc & 0x0008 ? '1' : '0',
                        opc & 0x0004 ? '1' : '0',
                        opc & 0x0002 ? '1' : '0',
                        opc & 0x0001 ? '1' : '0' );  
                            
        mnemonic = m_pic->opcodeToString( opc );  
      }
      else
      {
        opcode = "0000 : 00 0000 0000 0000 ";
        mnemonic = "nop";
      }  
      
      ui_view->setLine( i, opcode, mnemonic );
    }
    
    ui_view->viewport()->update();
  }
}

void
ROMView::refreshSLOT()
{
  static QString str;
  static int speedCnt = 0;
  
  if (!m_pic) return;
  if (!isVisible()) return;
  
  const long cycles = m_pic->cycles();
  str.sprintf( "%ld", cycles );
  ui_cycles->setText( str );
  
  str.sprintf( "%03X", m_pic->pc() );
  ui_pc->setText( str );

  speedCnt++;
  
  if (speedCnt >= 10)
  {
    speedCnt = 0;

    str.sprintf( "%.2f%%", m_pic->percentSpeed() );
    ui_speed->setText( str );
  }
  
  m_lastPC = m_pic->pc();
  
  ui_view->viewport()->update();
}
