//======================================================================
// File:		romviewtable.h
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#ifndef ROMVIEWTABLE_HH
#define ROMVIEWTABLE_HH

#include <qscrollview.h>
#include <qimage.h>
#include <qpainter.h>
#include <qheader.h>

class Pic;
class QPixmap;
class QPopupMenu;

class ROMViewTableHeader : public QHeader
{
  Q_OBJECT
public:
  enum Sections
  {
    PC=0, 
    OPCODE,
    MNEMONIC,
    READ,
    AVG,
    LAST,
    MIN,
    MAX,
    NumSections
  };
    
  ROMViewTableHeader( QWidget *parent=0, const char *name=0 );
  virtual ~ROMViewTableHeader();
  
protected:
  QPopupMenu *m_popup;

  void mousePressEvent( QMouseEvent * );
  
protected slots:
  void popupSLOT( int );
  
signals:
  void setVisibility( int, bool );

};

class ROMViewTable : public QScrollView
{
  Q_OBJECT
public:
  enum 
  {
    SetBreakpoint,
    RemoveBreakpoint,
    EditBreakpoint,
    ActivateBreakpoint,
    DeactivateBreakpoint,
    JumpHere,
    RunHere
  };
    
  ROMViewTable( QWidget *parent=0, const char *name=0 );
  virtual ~ROMViewTable();
  
  void setLine( int line, const QString & opcode, const QString & mnemonic );
  void setPic( Pic * );
  
  int lineHeight() const;
  
public slots:
  void setStatisticsColorsSLOT( const QColor & low, const QColor & medium,
                                const QColor & high, const QColor & highest );
  void setFixedFontSLOT( const QFont & );
  void setColumnVisibilitySLOT( int col, bool show );
  
signals:
  void refresh();
  void showBreakpoint( int );
  void newBreakpoint( int );
  void removeBreakpoint( int );
  void changedBreakpoint( int );
  
protected:
  QString              m_opcode[4096];    
  QString              m_mnemonic[4096];    
  int                  m_numLines;      
  int                  m_current;       
  Pic                 *m_pic;           
  QPixmap             *m_bpPix;         
  QPixmap             *m_ibpPix;        
  QPixmap             *m_cbpPix;         
  QPixmap             *m_icbpPix;        
  QPixmap             *m_pcPix;         
  QPopupMenu          *m_popup;         
  ROMViewTableHeader  *m_header;
  QBrush               m_brush[4];       
  QBrush               m_brushDark[4];   
  QString              m_tmpText;        
  bool                 m_visible[ROMViewTableHeader::NumSections];
  
  void drawContents( QPainter *, int, int, int, int );
  void contentsMousePressEvent( QMouseEvent * );
  void contentsMouseMoveEvent( QMouseEvent * );
  void contentsMouseDoubleClickEvent( QMouseEvent * );
  void resizeEvent( QResizeEvent * );
  
protected slots:
  void popupSLOT( int );
  void headerResizedSLOT();
  
private:
  QFont m_fixedFont;

  void eraseBg( QPainter *p, int section,
                int y, int lh, const QBrush & );
  void drawText( QPainter *p, const QRect & clipRect, int section, 
                 int y, int lh, int align, 
                 const QString & text, const QBrush & bg );
  void drawInt( QPainter *p, const QRect & clipRect, int section, 
                int y, int lh, int align, 
                int value, const QBrush & bg );
  void drawDouble( QPainter *p, const QRect & clipRect, int section, 
                   int y, int lh, int align, 
                   double value, const QBrush & bg );
  
};

inline void
ROMViewTable::eraseBg( QPainter *p, int section, int y, int lh, 
                       const QBrush & bg )
{
  const QRect tmpRect( m_header->sectionPos( section ), y, 
                       m_header->sectionSize( section )-1, lh );
  
  p->fillRect( tmpRect, bg );
  const int x = tmpRect.x() + tmpRect.width();
  p->drawLine( x, y, x, y+lh ); 
}

inline void
ROMViewTable::drawText( QPainter *p, const QRect & clipRect, 
                        int section, int y, int lh, int align,
                        const QString & text, const QBrush & bg )
{
  const QRect tmpRect( m_header->sectionPos( section ), y, 
                       m_header->sectionSize( section )-1, lh );
  
  if (clipRect.intersects( tmpRect ))
  {
    p->fillRect( tmpRect, bg );
    const int x = tmpRect.x() + tmpRect.width();
    p->drawLine( x, y, x, y+lh );     
    
    p->drawText( tmpRect.x()+4, tmpRect.y(), 
                 tmpRect.width()-8, tmpRect.height(),
                 align, text );
  }
}

inline void
ROMViewTable::drawInt( QPainter *p, const QRect & clipRect, 
                       int section, int y, int lh, int align,
                       int value, const QBrush & bg )
{
  const QRect tmpRect( m_header->sectionPos( section ), y, 
                       m_header->sectionSize( section )-1, lh );
  
  if (clipRect.intersects( tmpRect ))
  {
    p->fillRect( tmpRect, bg );
    const int x = tmpRect.x() + tmpRect.width();
    p->drawLine( x, y, x, y+lh );     
    
    m_tmpText.setNum( value );
    p->drawText( tmpRect.x()+4, tmpRect.y(), 
                 tmpRect.width()-8, tmpRect.height(),
                 align, m_tmpText );
  }
}

inline void
ROMViewTable::drawDouble( QPainter *p, const QRect & clipRect, 
                          int section, int y, int lh, int align,
                          double value, const QBrush & bg )
{
  const QRect tmpRect( m_header->sectionPos( section ), y, 
                       m_header->sectionSize( section )-1, lh );
  
  if (clipRect.intersects( tmpRect ))
  {
    p->fillRect( tmpRect, bg );
    const int x = tmpRect.x() + tmpRect.width();
    p->drawLine( x, y, x, y+lh );     
    
    m_tmpText.setNum( value, 'g', 4 );
    p->drawText( tmpRect.x()+4, tmpRect.y(), 
                 tmpRect.width()-8, tmpRect.height(),
                 align, m_tmpText );
  }
}

#endif // ROMVIEWTABLE_HH
