//======================================================================
// File:		sevesegmentdisplay.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <qpainter.h>

#include <picregister.h>
#include <sevensegmentdisplay.h>

SevenSegmentDisplay::SevenSegmentDisplay( const QString & name ) :
  BreadBoardPart( name ),
  m_segLength( 48 ),
  m_segWidth( 16 )
{
  m_rect = QRect( 0, 0, 
                  m_segLength+2*m_segWidth+2+8+32,
                  2*m_segLength+1+8+m_segWidth+16+16 );
  
  for (int i=0; i<8; ++i) 
  {
    m_register[i] = 0;
    m_value[i] = 0;
  }
  
  m_segmentColor = Qt::red;
    
  for (int i=0; i<7; ++i)
  {
    addPin( i, 0, 16+16*i, 
            BreadBoardPartPin::ConnectRight, QString( QChar( 'a'+i ) ) );
  }
  
  int w2 = m_rect.width()/2;
  w2 = w2 - (w2%16);
  
  addPin( 7, w2, m_rect.height()-16, BreadBoardPartPin::ConnectTop, "A" );
  
  startTimer( 10 );
}

SevenSegmentDisplay::~SevenSegmentDisplay()
{
}

void
SevenSegmentDisplay::paint( QPainter *p )
{ 
  p->fillRect( 0, 0, rect().width(), rect().height(), Qt::white );
  
  if (selected())
  {
    p->setPen( Qt::red );
  }
  else
  {
    p->setPen( Qt::black );
  }
  
  p->drawRect( 16, 0, rect().width()-16, rect().height()-16 );
  
  p->setPen( NoPen );
  
  p->setBrush( segmentColor(0) );
  p->save();
  p->translate( 32+4+m_segWidth+1, 4 );
  p->drawRect( 0, 0, m_segLength, m_segWidth );
  p->setPen( Qt::white );
  p->drawText( 0, 0, m_segLength, m_segWidth, Qt::AlignCenter, "a" );
  p->restore();
  
  p->setBrush( segmentColor(1) );
  p->save();
  p->translate( 32+3+m_segWidth, 4+m_segWidth/2 );
  p->rotate( 90 );
  p->drawRect( 0, 0, m_segLength, m_segWidth );
  p->setPen( Qt::white );
  p->drawText( 0, 0, m_segLength, m_segWidth, Qt::AlignCenter, "b" );
  p->restore();
  
  p->setBrush( segmentColor(2) );
  p->save();
  p->translate( 32+5+2*m_segWidth+m_segLength, 4+m_segWidth/2 );
  p->rotate( 90 );
  p->drawRect( 0, 0, m_segLength, m_segWidth );
  p->setPen( Qt::white );
  p->drawText( 0, 0, m_segLength, m_segWidth, Qt::AlignCenter, "c" );
  p->restore();
  
  p->setBrush( segmentColor(3) );
  p->save();
  p->translate( 32+5+m_segWidth, m_segWidth/2+m_segLength-2 );
  p->drawRect( 0, 0, m_segLength, m_segWidth );
  p->setPen( Qt::white );
  p->drawText( 0, 0, m_segLength, m_segWidth, Qt::AlignCenter, "d" );
  p->restore();
  
  p->setBrush( segmentColor(4) );
  p->save();
  p->translate( 32+3+m_segWidth, 5+m_segWidth/2+m_segLength );
  p->rotate( 90 );
  p->drawRect( 0, 0, m_segLength, m_segWidth );
  p->setPen( Qt::white );
  p->drawText( 0, 0, m_segLength, m_segWidth, Qt::AlignCenter, "e" );
  p->restore();
  
  p->setBrush( segmentColor(5) );
  p->save();
  p->translate( 32+5+2*m_segWidth+m_segLength, 5+m_segWidth/2+m_segLength );
  p->rotate( 90 );
  p->drawRect( 0, 0, m_segLength, m_segWidth );
  p->setPen( Qt::white );
  p->drawText( 0, 0, m_segLength, m_segWidth, Qt::AlignCenter, "f" );
  p->restore();
  
  p->save();
  p->setBrush( segmentColor(6) );
  p->translate( 32+5+m_segWidth, 2*m_segLength+m_segWidth/2-2 );
  p->drawRect( 0, 0, m_segLength, m_segWidth );  
  p->setPen( Qt::white );
  p->drawText( 0, 0, m_segLength, m_segWidth, Qt::AlignCenter, "g" );
  p->restore();
  
  drawPins( p );
}

QColor
SevenSegmentDisplay::segmentColor( int i )
{
  if (!m_register[i]) return Qt::white;
  
  if (m_value[i] & (0x01 << i))
  {
    m_value[i] = 0;
    return m_segmentColor;
  }
  
  m_value[i] = 0;
  return m_segmentColor.dark(200);
}

void
SevenSegmentDisplay::assignRegister( int i, PicRegister *reg, int bit )
{
  m_register[i] = reg;
  m_bit[i] = bit;
}
  
void
SevenSegmentDisplay::timerEvent( QTimerEvent * )
{
  for (int i=0; i<8; ++i)
  {
    if (m_register[i])
      m_value[i] |= m_register[i]->valuePic();
  }
}

