//======================================================================
// File:		stackview.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <qlabel.h>

#include <pic.h>
#include <stackview.h>

StackView::StackView( QWidget *parent, const char *name ) :
  UIStackView( parent, name ),
  m_stackPointer( -1 )
{
  m_text[0] = ui_0;
  m_text[1] = ui_1;
  m_text[2] = ui_2;
  m_text[3] = ui_3;
  m_text[4] = ui_4;
  m_text[5] = ui_5;
  m_text[6] = ui_6;
  m_text[7] = ui_7;
  
  startTimer( 100 );
  
  for (int i=0; i<8; ++i)
  {
    m_stack[i] = 0;
  }
}

StackView::~StackView()
{
}

void
StackView::timerEvent( QTimerEvent * )
{
  refreshSLOT();
}

void
StackView::setPic( Pic * pic )
{
  m_pic = pic;
  
  refreshSLOT();
}

void
StackView::refreshSLOT()
{
  if (!m_pic) return;
  
  QString str;
  const QColorGroup cg = colorGroup();
  
  for (int i=0; i<m_pic->stack()->size(); ++i)
  {
    const int newStack = m_pic->stack()->at(i);
    
    if (newStack != m_stack[i])
    {
      str.sprintf( "%04X - %04X", newStack, m_pic->stack()->callAt(i) );
      m_text[i]->setText( str );  
      
      m_stack[i] = newStack;
    }
  } 
  
  if (m_pic->stack()->stackPointer() != m_stackPointer)
  {
    if (m_stackPointer >= 0)
    {
      m_text[m_stackPointer]->setBackgroundColor( cg.background() );
    }
    
    const int newStackPointer = m_pic->stack()->stackPointer();
    
    m_text[newStackPointer]->setBackgroundColor( cg.base() );
    m_stackPointer = newStackPointer;
  }
}
