//======================================================================
// File:		stringconversion.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <stringconversion.h>

bool StringConversion::s_initialized = false;
QString StringConversion::s_binString[256];
QString StringConversion::s_hexString[256];
QString StringConversion::s_decString[256];
char *StringConversion::s_unitString[3] = { " Hz", " kHz", " MHz" };

QString
StringConversion::binString( int value )
{
  if (!s_initialized) initStrings();
  
  return s_binString[value & 0xff];
}

QString
StringConversion::hexString( int value )
{
  if (!s_initialized) initStrings();
  
  return s_hexString[value & 0xff];
}

QString
StringConversion::decString( int value )
{
  if (!s_initialized) initStrings();
  
  return s_decString[value & 0xff];
}

QString 
StringConversion::freqString( double freq )
{
  int unit = 0;
  
  if (freq >= 1000.0)
  {
    freq /= 1000.0;
    unit = 1;
  }
  if (freq >= 1000.0)
  {
    freq /= 1000.0;
    unit = 2;
  }
  
  QString str;
  str.sprintf( "%.2f%s", freq, s_unitString[unit] );
  
  return str;
}

QString 
StringConversion::freqString_g( double freq )
{
  int unit = 0;
  
  if (freq >= 1000.0)
  {
    freq /= 1000.0;
    unit = 1;
  }
  if (freq >= 1000.0)
  {
    freq /= 1000.0;
    unit = 2;
  }
  
  QString str;
  str.sprintf( "%g%s", freq, s_unitString[unit] );
  
  return str;
}

void
StringConversion::initStrings()
{
  for (int i=0; i<256; ++i)
  {
    s_binString[i].sprintf( "%c%c%c%c%c%c%c%c",
                            (i & 0x80) ? '1' : '0',
                            (i & 0x40) ? '1' : '0',
                            (i & 0x20) ? '1' : '0',
                            (i & 0x10) ? '1' : '0',
                            (i & 0x08) ? '1' : '0',
                            (i & 0x04) ? '1' : '0',
                            (i & 0x02) ? '1' : '0',
                            (i & 0x01) ? '1' : '0' );
    
    s_hexString[i].sprintf( "%02X", i );
    s_decString[i].sprintf( "%d", i );
  }
  
  s_initialized = true;
}
