//======================================================================
// File:		symbol.h
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#ifndef SYMBOL_HH
#define SYMBOL_HH

#include <qstring.h>

/** Encapsulation of a symbol from the .cod file.

    Symbol may be Address or Constant. Address is 16 bit, constant 8 bit.
  */
class Symbol
{
public:
  enum Type
  {
    Address,
    Constant
  };
  
    /// ctor
  Symbol( const QString & name, int value, Type type );
    /// dtor
  ~Symbol();

  /// retrieve symbol name
  QString name() const { return m_name; }
  /// retrieve symbol value
  int value() const { return m_value; }
  /** retrieve symbol type
      May be Symbol::Address or Symbol::Constant
    */
  Type type() const { return m_type; }
    
  bool isAddress() const { return (m_type == Address); }
  bool isConstant() const { return (m_type == Constant); }
  
protected:
  QString m_name;
  int     m_value;
  Type    m_type;
  
};

#endif // SYMBOL_HH
