//======================================================================
// File:		symboltable.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <symboltable.h>

SymbolTable::SymbolTable()
{
  m_address = new SymbolDict ( 1009, false );
  m_address->setAutoDelete( true );
  
  m_constant = new SymbolDict ( 1009, false );
  m_constant->setAutoDelete( true );
}

SymbolTable::~SymbolTable()
{
  delete m_address;
  delete m_constant;
}

void
SymbolTable::clear()
{
  m_address->clear();
  m_constant->clear();
}

void
SymbolTable::addSymbol( Symbol *symbol )
{
  if (symbol->isAddress())
  {
    m_address->insert( symbol->name().latin1(), symbol );
  }
  else
  {
    m_constant->insert( symbol->name().latin1(), symbol );
  }
}

Symbol *
SymbolTable::constant( const QString & name ) const
{
  return m_constant->find( name.latin1() );
}

Symbol *
SymbolTable::address( const QString & name ) const
{
  return m_address->find( name.latin1() );
}

SymbolIterator
SymbolTable::addressesIterator()
{
  return SymbolIterator( *m_address );   
}

SymbolIterator
SymbolTable::constantsIterator()
{
  return SymbolIterator( *m_constant );   
}
