//======================================================================
// File:		symboltable.h
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#ifndef SYMBOLTABLE_HH
#define SYMBOLTABLE_HH

#include <qasciidict.h>

#include <symbol.h>

typedef QAsciiDict<Symbol> SymbolDict;
typedef QAsciiDictIterator<Symbol> SymbolIterator;

class SymbolTable
{
public:
  SymbolTable();
  ~SymbolTable();
  
  int numConstant() const;
  int numAddress() const;
  
  void clear();
  
  void addSymbol( Symbol * );
  
  Symbol *constant( const QString & ) const;
  Symbol *address( const QString & ) const;
  
  SymbolIterator addressesIterator();
  SymbolIterator constantsIterator();
  
protected:
  SymbolDict *m_address;
  SymbolDict *m_constant;
  
};

inline int 
SymbolTable::numConstant() const
{
  return m_constant->count();
}

inline int 
SymbolTable::numAddress() const
{
  return m_address->count();
}

#endif // SYMBOLTABLE_HH
