//======================================================================
// File:		symbolview.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <qlabel.h>
#include <qpushbutton.h>
#include <qlistview.h>
#include <qpopupmenu.h>
#include <qcursor.h>

#include <symbolview.h>
#include <symbolviewitem.h>
#include <pic.h>
#include <stringconversion.h>

SymbolView::SymbolView( QWidget *parent, const char *name ) :
  UISymbolView( parent, name ),
  m_pic( 0 )
{
  m_popup = new QPopupMenu( this );
  
  connect( m_popup, SIGNAL( activated( int ) ),
           this, SLOT( popupSLOT( int ) ));
  
  connect( ui_register, SIGNAL( selectionChanged( QListViewItem * )),
           this, SLOT( selectionChangedSLOT( QListViewItem * )));
  connect( ui_register, SIGNAL( rightButtonPressed ( QListViewItem *, const QPoint &, int )),
           this, SLOT( registerContextSLOT( QListViewItem * ) ));
}

SymbolView::~SymbolView()
{
}

void
SymbolView::setPic( Pic *pic )
{
  m_pic = pic;
  
  SymbolIterator addrIt = m_pic->symbolTable()->addressesIterator();
  SymbolIterator constIt = m_pic->symbolTable()->constantsIterator();
  
  ui_addresses->clear();
  ui_constants->clear();
  ui_register->clear();
  
  for ( ; addrIt.current(); ++addrIt)
  {
    QString str;
    str.sprintf( "%03X", addrIt.current()->value() );
    
    new SymbolViewItem( ui_addresses, addrIt.current()->name(), str );
  }
   
  for ( ; constIt.current(); ++constIt)
  {
    QString str;
    
    // retrieve register (if any)
    // later we can GUESS if the constant is meant to be a register
    //
    PicRegister *reg = 0;
    
    if (constIt.current()->value() <= 0x1ff)
    {
      reg = m_pic->ram()->get( constIt.current()->value() );
    }
       
    // It's an address
    // 
    if (constIt.current()->value() == 
        m_pic->ram()->symbolToAddr( constIt.current()->name() ))
    {
      str.sprintf( "%03X", constIt.current()->value() );
      new SymbolViewItem( ui_register, constIt.current()->name(), str );
    }
    // GUESS if it's a general purpose register
    //
    else if (reg && reg->type() == PicRegister::GeneralPurpose)
    {
      str.sprintf( "%03X", constIt.current()->value() );
      new SymbolViewItem( ui_register, constIt.current()->name(), str );
    }
    // must be a constant
    //
    else
    {
      new SymbolViewItem( ui_constants, constIt.current()->name(), 
                 StringConversion::hexString( constIt.current()->value() ) );
    }
  } 
}

void
SymbolView::selectionChangedSLOT( QListViewItem *item )
{
  int addr = item->text(1).toInt( 0, 16 );
  
  emit registerSelected( addr );
}

void
SymbolView::registerContextSLOT( QListViewItem *item )
{
  int addr = item->text(1).toInt( 0, 16 );
  
  m_popup->clear();
  
  m_selectedReg = m_pic->ram()->get(addr);
  
  if (m_selectedReg->watched())
  {
    QString str;
    str.sprintf( tr("Remove %s from Watch"), item->text(0).latin1() );
    m_popup->insertItem( str, ID_REMOVE );
  } 
  else 
  {
    QString str;
    str.sprintf( tr("Add %s to Watch"), item->text(0).latin1() );
    m_popup->insertItem( str, ID_ADD );
  }  
  
  m_popup->popup( QCursor::pos() );
}

void
SymbolView::popupSLOT( int id )
{
  if (ID_REMOVE == id)
  {
    emit removeWatch( m_selectedReg );
  }
  else if (ID_ADD == id)
  {
    emit addWatch( m_selectedReg );
  }
}
