//======================================================================
// File:		symbolviewitem.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <qpainter.h>

#include <symbolviewitem.h>

SymbolViewItem::SymbolViewItem( QListView *list, const QString & name,
                                const QString & value ) :
  QListViewItem( list, name, value )
{
}

SymbolViewItem::~SymbolViewItem()
{
}

void
SymbolViewItem::paintCell( QPainter *p, const QColorGroup & cg,
                           int col, int width, int align )
{
  p->save();
  
  if ( 0==col )
  {
    QFont font = p->font();
    font.setBold( true );
    p->setFont( font );
  }
  
  if (isSelected())
  {
    p->fillRect( 0, 0, width, height(), cg.highlight() );
    p->setPen( cg.highlightedText() );    
    p->drawText( 4, 0, width-8, height(), align, text( col ) );
    p->setPen( cg.mid() );
    p->drawLine( width-1, 0, width-1, height() );
    p->drawLine( 0, height()-1, width, height()-1 );
  }
  else
  {
    int pos = itemPos() / height();
    
    if (pos & 1)
    {
      p->fillRect( 0, 0, width, height(), cg.base().dark( 105 ) );
    }
    else
    {
      p->fillRect( 0, 0, width, height(), cg.base() );
    }
    p->drawText( 4, 0, width-8, height(), align, text( col ) );
    p->setPen( cg.mid() );
    p->drawLine( width-1, 0, width-1, height() );
    p->drawLine( 0, height()-1, width, height()-1 );
  }
  
  p->restore();
}

int 
SymbolViewItem::width( const QFontMetrics & fm, const QListView *list, 
                       int col ) const
{
  if ( 0==col )
  {
    QFont font = list->font();
    font.setBold( true );
    QFontMetrics tmpFm( font );
    
    return tmpFm.width( text(col) ) + 8;
  }
    
  return fm.width( text(col) ) + 8;
}
