//======================================================================
// File:		textedit.h
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#ifndef TEXTEDIT_HH
#define TEXTEDIT_HH

#include <qtextedit.h>

class QPixmap;
class Pic;
class QLabel;
class PicRegister;

class TextEdit : public QTextEdit
{
  Q_OBJECT
public:
  // put some insane id's here otherwise
  // the QTextEdit does some weird crap
  // missinterpreting it (No hints in the docs)
  //
  enum
  {
    ID_ADD_BREAKPOINT = 1111,
    ID_REMOVE_BREAKPOINT,
    ID_EDIT_BREAKPOINT,
    ID_ACTIVATE_BREAKPOINT,
    ID_DEACTIVATE_BREAKPOINT,
    ID_JUMP_HERE,
    ID_RUN_HERE,
    ID_FINISH_CALL,
    ID_ADD_TO_WATCH,
    ID_REMOVE_FROM_WATCH,
    ID_RESET,
    ID_RUN,
    ID_ANIMATE,
    ID_STEP,
    ID_STEP_OVER,
    ID_STOP,
    ID_FOLLOW_PC,
    ID_FIND_PC,
    ID_FIND_ADDRESS
  };
    
  TextEdit( QWidget *parent=0, const char *name=0 );
  virtual ~TextEdit();

  bool autoIndent() const { return m_autoIndent; }
  void setPc( int pc );
  void resetPcToLineTable();
  void setLine( int addr, int line ) { m_addrToLine[addr] = line; }
  int lineForPc( int addr ) const { return m_addrToLine[addr]; }
  
  void setPic( Pic *pic ) { m_pic = pic; }
  
  QRect tip( const QPoint & pos, QString & text );
  
signals:
  void newBreakpointRequest( int );
  void removeBreakpointRequest( int );
  void editBreakpointRequest( int );
  void jumpHereRequest( int );
  void runHereRequest( int );
  void addToWatchRequest( PicRegister * );
  void removeFromWatchRequest( PicRegister * );
  void resetRequest();
  void runRequest();
  void finishCallRequest();
  void animateRequest();
  void stepRequest();
  void stepOverRequest();
  void stopRequest();
  void changedBreakpoint( int );
  
public slots:
  void setAutoIndentSLOT( bool ai ) { m_autoIndent = ai; }

protected:
  bool     m_autoIndent;
  int      m_lh;
  int      m_pcLine;
  QPixmap *m_bpPix;                     
  QPixmap *m_ibpPix;                    
  QPixmap *m_cbpPix;                     
  QPixmap *m_icbpPix;                    
  QPixmap *m_pcPix;                     
  QPixmap *m_brownoutPix;
  QPixmap *m_animationPix;
  QPixmap *m_resetPix;
  QPixmap *m_runPix;
  QPixmap *m_stopPix;
  QPixmap *m_stop_after_commandPix;
  QPixmap *m_stop_after_procedurePix;
  int      m_popupAddr;
  QString  m_popupSymbolName;
  int      m_popupSymbolAddr;
  int      m_addrToLine[2048];
  int      m_pc;
  Pic     *m_pic;
  int      m_margin;
  bool     m_followPc;
  
  void paintEvent( QPaintEvent * );
  QPopupMenu *createPopupMenu( const QPoint & pos );
  int addrFromLine( int, int lastPc=0 ) const;
  void findPc();
  void findAddress( int addr );
  
  bool isSpace( const QChar & );
  int symbolAddrAtPos( const QPoint & pos );
  QString tokenAtPos( const QPoint & pos );
  int symbolAddrFromToken( const QString & token );
  
protected slots:
  void returnPressedSLOT();
  void popupSLOT( int );
  
};

inline int 
TextEdit::addrFromLine( int line, int lastAddr ) const
{
  for (int addr=lastAddr; addr<2048; ++addr)
  {
    if (m_addrToLine[addr] == line) return addr;
    if (m_addrToLine[addr] > line) return -1;
  }
  
  return -1;
}

inline bool
TextEdit::isSpace( const QChar & c )
{
  if (c.isSpace() || c == ',') return true;
  
  return false;
}

#endif // TEXTEDIT_HH
