//======================================================================
// File:		tools.h
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#ifndef TOOLS_HH
#define TOOLS_HH

#include <qglobal.h>
#include <qcolor.h>

//#define USE_CPP_FALLBACK

Q_INT32 tool_convert_8_bit( float value );
Q_INT32 tool_convert_10_bit( float value );
QRgb    tool_pixel_value( float red, float green, float blue );
float   tool_blend( float a, float b, float aFac, float bFac );
Q_INT32 tool_float_to_int_x_255( float val );
Q_INT32 tool_float_to_int( float val );
Q_INT32 tool_float_to_int_x_1023( float val );
float   tool_invert( float val );

inline Q_INT32
tool_convert_8_bit( float value ) 
{ 
  const Q_INT32 iValue = tool_float_to_int_x_255( value );
  
  if (iValue & 0x80000000) return 0;
  if (iValue & 0x7FFFFF00) return 255;
  
  return iValue;
}

inline Q_INT32
tool_convert_10_bit( float value ) 
{ 
  const Q_INT32 iValue = tool_float_to_int_x_1023( value );
  
  if (iValue & 0x80000000) return 0;
  if (iValue & 0x7FFFFC00) return 1023;
  
  return iValue;
}

inline QRgb
tool_pixel_value( float red, float green, float blue ) 
{ 
  return qRgb( tool_convert_8_bit( red ), 
               tool_convert_8_bit( green ), 
               tool_convert_8_bit( blue ) ); 
}

inline float
tool_blend( float a, float b, float aFac, float bFac )
{
  return (a*aFac + b*bFac);
}

inline Q_INT32
tool_float_to_int_x_255( float val ) 
{
#ifdef USE_CPP_FALLBACK
	return (int)(val * 255.0);
#else
  Q_INT32 b;
  static float fac = 255.0;

  __asm__ __volatile__ (
      "flds %1          \n\t"
      "fmul %2          \n\t"
      "fistpl %0        \n\t"
      : "=m" (b)
      : "m" (val), "m" (fac));

  return b;
#endif
}

inline Q_INT32
tool_float_to_int( float val ) 
{
#ifdef USE_CPP_FALLBACK
	return (int)(val * 255.0);
#else
  Q_INT32 b;
  __asm__ __volatile__ (
      "flds %1          \n\t"
      "fistpl %0        \n\t"
      : "=m" (b)
      : "m" (val));

  return b;
#endif
}

inline Q_INT32
tool_float_to_int_x_1023( float val ) 
{
#ifdef USE_CPP_FALLBACK
	return (int)(val * 1023.0);
#else
  Q_INT32 b;
  static float fac = 1023.0;

  __asm__ __volatile__ (
      "flds %1          \n\t"
      "fmul %2          \n\t"
      "fistpl %0        \n\t"
      : "=m" (b)
      : "m" (val), "m" (fac));

  return b;
#endif
}

inline float
tool_invert( float value )
{
  return 1.0f-value;
}

#endif // TOOLS_HH
