//======================================================================
// File:		watchcondition.h
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#ifndef WATCHCONDITION_HH
#define WATCHCONDITION_HH

class PicRegister;

class WatchCondition
{
public:
  enum Mode
  {
    Unknown=0,
    OnRead,
    OnWrite,
    Equal,
    NotEqual,
    GreaterThan,
    GreaterOrEqual,
    LessThan,
    LessOrEqual,
    And
  };
    
  enum AccessMode
  {
    Read = 0,
    Write,
    InternalWrite
  };
    
  WatchCondition();
  ~WatchCondition();
  
  bool match( int regValue, WatchCondition::AccessMode ) const;
  
  void setCondition( WatchCondition::Mode, int value );
  
  bool enabled() const { return m_enabled; }
  void setEnabled( bool enabled ) { m_enabled = enabled; }
  
protected:
  Mode  m_mode;
  int   m_value;
  bool  m_enabled;
  
};
  
inline bool
WatchCondition::match( int regValue, AccessMode accessMode ) const
{
  if (!enabled()) return false;
  
  if (accessMode != Read)
  {
    switch (m_mode)
    {
    case GreaterThan:
      return (regValue > m_value);
    case GreaterOrEqual:
      return (regValue >= m_value);
    case LessThan:
      return (regValue < m_value);
    case LessOrEqual:
      return (regValue <= m_value);
    case Equal:
      return (regValue == m_value);
    case NotEqual:
      return (regValue != m_value);
    case OnRead:
      return false;
    case OnWrite:
      return (accessMode == Write);
    case And:
      return ((regValue & m_value) != 0);
    default:
      return false;
    }
  }
  
  return (m_mode == OnRead);
}

#endif // WATCHCONDITION_HH
