//======================================================================
// File:		watchconditionedit.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <watchconditionedit.h>

WatchConditionEdit::WatchConditionEdit( QWidget *parent, const char *name ) :
  QLineEdit( parent, name )
{
  connect( this, SIGNAL( textChanged( const QString & ) ),
           this, SLOT( textChangedSLOT( const QString & )));
}

WatchConditionEdit::~WatchConditionEdit()
{
}

void
WatchConditionEdit::textChangedSLOT( const QString & text )
{
  QString pattern = text.simplifyWhiteSpace().stripWhiteSpace().upper();
  
  if (pattern.isEmpty())
  {
    setOk( true );
    m_mode = WatchCondition::Unknown;
    m_value = 0;
    return;
  }
  
  QString command;
  QString compare;
  QString value;
  
  unsigned pos = 0;
  // get command (if any)
  //
  while (pos < pattern.length() && 
         !isCondition( pattern.constref(pos) ) &&
         !pattern.constref(pos).isSpace())
  {
    command += pattern.constref(pos);
    ++pos;
  }
  
  if (!command.isEmpty())
  {
    if ((command != "READ" && command != "WRITE") ||
        pos < pattern.length())
    {
      setOk( false );
      return;
    }
    
    if (command == "READ") setMode( WatchCondition::OnRead );
    else                   setMode( WatchCondition::OnWrite );
    
    setOk( true );
    return;
  }
  
  skipWhiteSpace( &pattern, &pos );
  
  // get condition
  //
  while (pos < pattern.length() && 
         isCondition( pattern.constref(pos) ))
  {
    compare += pattern.constref(pos);
    ++pos;
  }
  
  skipWhiteSpace( &pattern, &pos );
  
  // get condition value
  //
  while (pos < pattern.length() && 
         isHexNumber( pattern.constref(pos) ))
  {
    value += pattern.constref(pos);
    ++pos;
  }
  
  m_value = -1;
  
  if (!value.isEmpty())
  {
    bool ok=false;
    m_value = value.toInt( &ok, 16 ) & 0xff;
     
    if (!ok)  
    { 
      m_value = -1;
    }
  }
  
  if      (compare == "==") setMode( WatchCondition::Equal );
  else if (compare == "!=") setMode( WatchCondition::NotEqual );
  else if (compare == ">")  setMode( WatchCondition::GreaterThan );
  else if (compare == ">=") setMode( WatchCondition::GreaterOrEqual );
  else if (compare == "<")  setMode( WatchCondition::LessThan );
  else if (compare == "<=") setMode( WatchCondition::LessOrEqual );
  else if (compare == "&")  setMode( WatchCondition::And );
  else                      setMode( WatchCondition::Unknown );
  
  if (compare.isEmpty() || m_value == -1)
  {
    //std::cerr << "first case" << std::endl;
    setOk( false );
    return;
  }
  
  if (mode() == WatchCondition::Unknown || m_value == -1)
  {
    //std::cerr << "second case" << std::endl;
    setOk( false );
    return;
  }
  
  setOk( true );
}

void
WatchConditionEdit::setOk( bool ok )
{
  if (ok)
  {
    setBackgroundColor( colorGroup().base() );
  }
  else
  {
    int red   = colorGroup().base().red();
    int green = colorGroup().base().green() - 20;
    int blue  = colorGroup().base().blue() - 20;
    
    if (green < 0) green = 0;
    if (blue < 0)  blue = 0;
    
    setBackgroundColor( QColor( red, green, blue ) );
  }
}
