//======================================================================
// File:		watchview.cpp
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#include <qheader.h>
#include <qtable.h>
#include <qtable.h>

#include <bintableitem.h>
#include <dectableitem.h>
#include <hextableitem.h>
#include <labeltableitem.h>
#include <enabletableitem.h>
#include <pic.h>
#include <watchview.h>
#include <watchcondition.h>
#include <watchbreaktableitem.h>

WatchObject::WatchObject( PicRegister *reg, int row )  :
  m_reg( reg ), m_row( row ) 
{
  m_watch = new WatchCondition;
  
  m_reg->setWatchCondition( m_watch );
}

WatchObject::~WatchObject()
{
  m_reg->setWatchCondition( 0 );
  
  delete m_watch;
}

WatchView::WatchView( QWidget *parent, const char *name ) :
  UIWatchView( parent, name ),
  m_pic( 0 )
{
  m_watches.setAutoDelete( true );
  
  ui_table->setLeftMargin( 0 );
  ui_table->verticalHeader()->hide();
 
  ui_table->setColumnReadOnly( 0, true );
  ui_table->setColumnReadOnly( 4, true );
  ui_table->setColumnReadOnly( 5, true );
  
  QHeader *hheader = ui_table->horizontalHeader();
  //hheader->setResizeEnabled( false );
  //hheader->setClickEnabled( false );
  
  int hw = fontMetrics().width( "XXX" ) + 16;  
  hheader->resizeSection( 1, hw );
  hheader->resizeSection( 2, hw );
  
  hw = fontMetrics().width( "00000000" ) + 16;
  hheader->resizeSection( 3, hw );
 
  hw = fontMetrics().width( "ABCDEFGHIJKLMOP" );
  hheader->resizeSection( 6, hw );
  
  connect( ui_table, SIGNAL( valueChanged( int, int ) ),
           this, SLOT( valueChangedSLOT( int, int ) )); 
  connect( ui_table, SIGNAL( clicked( int, int, int, const QPoint & ) ),
           this, SLOT( clickedSLOT( int, int, int, const QPoint & ) ));
}

WatchView::~WatchView()
{
}

void
WatchView::show()
{
  UIWatchView::show();
  refreshSLOT();
}

void
WatchView::setPic( Pic *pic )
{
  m_pic = pic;
  
  refreshSLOT();
}

void
WatchView::refreshSLOT()
{
  static QString str;
  
  if (!isVisible()) return;
  
  for (WatchObjectListIterator it( m_watches ); *it; ++it)
  {
    HexTableItem *hi = (HexTableItem *)ui_table->item( (*it)->row(), 1 );
    hi->update();
    ui_table->updateCell( (*it)->row(), 1 );
    
    DecTableItem *di = (DecTableItem *)ui_table->item( (*it)->row(), 2 );
    di->update();
    ui_table->updateCell( (*it)->row(), 2 );
    
    BinTableItem *bi = (BinTableItem *)ui_table->item( (*it)->row(), 3 );
    bi->update();
    ui_table->updateCell( (*it)->row(), 3 );
  
    str.setNum( hi->reg()->freezedReadCnt() );
    ui_table->setText( (*it)->row(), 4, str );
  
    str.setNum( hi->reg()->freezedWriteCnt() );
    ui_table->setText( (*it)->row(), 5, str );
  }
}

void
WatchView::valueChangedSLOT( int row, int col )
{
  std::cerr << "changed " << row << " " << col << std::endl;
  
  if (col >= 1 && col <= 3)
  {
    PicTableItem *item = 
        dynamic_cast<PicTableItem *> (ui_table->item( row, col ));
    
    if (item)
    {
      const int value = item->value();

      for (int i=1; i<=3; ++i)
      {
        item = (PicTableItem *)ui_table->item( row, i );
        item->setValue( value );
        item->update();
        ui_table->updateCell( row, i );
      }

      emit updateViews();
    }
  } 
  else if (col == 6)
  {
    if (!ui_table->text( row, col ).simplifyWhiteSpace().stripWhiteSpace().isEmpty())
    {
      EnableTableItem *check = dynamic_cast<EnableTableItem *> (ui_table->item( row, 7 ));
      
      if (check)
      {
        WatchObject *obj = findObjectForRow( row );
        if (obj) obj->watch()->setEnabled( true );
        
        check->setChecked( true );
      }
    }
  }
}

WatchObject *
WatchView::findObjectForRow( int row )
{
  for (WatchObjectListIterator it( m_watches ); *it; ++it)
  {
    if ((*it)->row() == row)
    {
      return (*it);
    }
  }
  
  return 0;
}

void
WatchView::addWatchSLOT( PicRegister *reg )
{
  QString str;
  
  WatchObject *obj = new WatchObject( reg, ui_table->numRows() );
  
  m_watches.append( obj );
  ui_table->setNumRows( ui_table->numRows()+1 );
  
  ui_table->setItem( obj->row(), 0, 
                     new LabelTableItem( ui_table, reg->name(),
                                         AlignLeft|AlignVCenter, true ) );
  ui_table->setItem( obj->row(), 1, 
                     new HexTableItem( ui_table, reg ));
  ui_table->setItem( obj->row(), 2, 
                     new DecTableItem( ui_table, reg ));
  ui_table->setItem( obj->row(), 3, 
                     new BinTableItem( ui_table, reg ));
    
  str.setNum( reg->freezedReadCnt() );
  ui_table->setItem( obj->row(), 4, 
                     new LabelTableItem( ui_table, str, 
                                         AlignRight|AlignVCenter ));
  str.setNum( reg->freezedWriteCnt() );
  ui_table->setItem( obj->row(), 5, 
                     new LabelTableItem( ui_table, str, 
                                         AlignRight|AlignVCenter ));
  
  ui_table->setItem( obj->row(), 6,
                     new WatchBreakTableItem( ui_table, obj->watch() ) );
  
  ui_table->setItem( obj->row(), 7, new EnableTableItem( ui_table, tr("Enabled") ) );
  
  refreshSLOT();
}

void
WatchView::removeWatchSLOT( PicRegister * )
{
}

void
WatchView::setEditable( bool /*editable*/ )
{
  //ui_table->setReadOnly( !editable );
}

void
WatchView::clickedSLOT( int row, int col, int button, const QPoint & )
{
  if (col == 7 && button == LeftButton)
  {
    WatchObject *obj = findObjectForRow( row );
    
    if (obj)
    {
      EnableTableItem *check = dynamic_cast<EnableTableItem *> (ui_table->item( row, 7 ));
      
      if (check)
      {
        check->setChecked( !check->isChecked() );
        obj->watch()->setEnabled( check->isChecked() );
      }
    }
  }
}

