//======================================================================
// File:		workspace.h
// Author:	Matthias Toussaint
// Created:	Mon Jul 26 23:08:00 CET 2004
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2004 Matthias Toussaint
//======================================================================

#ifndef WORKSPACE_HH
#define WORKSPACE_HH

#include <qworkspace.h>

#include <pic.h>

class BreadboardView;
class BreakpointView;
class CPUView;
class ClockView;
class Editor;
class QPixmap;
class RAMView;
class ROMView;
class StackView;
class WatchView;
class SymbolView;
class InstructionTraceView;

class Workspace : public QWorkspace
{
  Q_OBJECT
public:
  Workspace( QWidget *parent=0, const char *name=0 );
  virtual ~Workspace();
  
  double clockSpeed() { return m_pic->clockSpeed(); }
  double crystalFrequency() { return m_pic->crystalFrequency(); }
  double percentSpeed() { return m_pic->percentSpeed(); }
  bool simulatorAnimation() const { return m_animateTimer != -1; }
  bool halted() const { return m_pic->halted() && m_animateTimer == -1; }
  void quit() { m_pic->quit(); m_pic->wait(); }
  Pic *pic() { return m_pic; }
  
  void freeze();
  
signals:
  void updateViews();
  void updateMenus( bool run );
  void message( const QString & );
  
public slots:
  void showRAMSLOT( bool );
  void showROMSLOT( bool );
  void showCPUSLOT( bool );
  void showStackSLOT( bool );
  void showEditorSLOT( bool );
  void showClockSLOT( bool );
  void showWatchSLOT( bool );
  void showBreakpointsSLOT( bool );
  void showSymbolsSLOT( bool );
  void showInstructionTraceSLOT( bool );
  void resetPicSLOT();
  void runPicSLOT();
  void stepPicSLOT();
  void overPicSLOT();
  void finishCallPicSLOT();
  void stopPicSLOT();
  void animatePicSLOT();
  void refreshSLOT();
  void showFindSLOT();
  void findAgainSLOT();
  void showReplaceSLOT();
  void replaceAgainSLOT();
  void gotoLineSLOT();
  void gotoSelectionSLOT();
  void cutSLOT();
  void copySLOT();
  void pasteSLOT();
  void selectAllSLOT();
  void undoSLOT();
  void redoSLOT();
  void makeSLOT();
  void saveSLOT();
  void openSLOT();
  void upperSLOT();
  void lowerSLOT();
  void freeRunSLOT( bool );
  void tracingSLOT( bool );
    
protected:
  RAMView         *m_ramView;
  ROMView         *m_romView;
  StackView       *m_stackView;
  CPUView         *m_cpuView;
  Editor          *m_editor;
  ClockView       *m_clockView;
  WatchView       *m_watchView;
  BreakpointView  *m_breakpointView;
  BreadboardView  *m_breadboardView;
  SymbolView      *m_symbolView;
  InstructionTraceView *m_instructionTraceView;
  Pic             *m_pic;
  QPixmap         *m_yapidePix;
  int              m_animateTimer;
  int              m_runUpdateTimer;
  
  void timerEvent( QTimerEvent * );
  
protected slots:
  void showBreakpointSLOT( int );
  void newBreakpointSLOT( int );
  void removeBreakpointSLOT( int );
  void changedBreakpointSLOT( int );
  void reloadPicSLOT();
  void addTraceSLOT( PicRegister * );
  void removeTraceSLOT( PicRegister * );
  
private:
  void connectRAM_Watch();
  void connectRAM_CPUInfo();
  void createWatchIfNeeded();
  void createBreakpointsIfNeeded();
  void createRAMIfNeeded();
  
  void setEditable( bool editable );
  
};

#endif // WORKSPACE_HH
